from datetime import datetime, timedelta

def calculate_leave_days(start_date, end_date, start_time_status, end_time_status):
    # Check that start_date and end_date are not on weekends
    if start_date.weekday() in [5, 6] or end_date.weekday() in [5, 6]:
        raise ValueError("Data Hahu Ou Data Remata Labele iha Sabado ou Domingo")
    
    hourly_values = {
        "08:00": 8.0,
        "13:00": 4.0,
        "12:00": 4.0,
        "17:00": 8.0,
    }
    
    if start_time_status not in hourly_values or end_time_status not in hourly_values:
        raise ValueError("Wrong input Start Time and End Time")
    
    total_days = 0
    if start_date == end_date:
        # Calculate number of hours based on start_time_status and end_time_status
        if start_time_status == "08:00" and end_time_status == "17:00":
            total_days = 1.0
        elif start_time_status == "13:00" and end_time_status == "17:00":
            total_days = 0.5
        elif start_time_status == "08:00" and end_time_status == "12:00":
            total_days = 0.5
        else:
            raise ValueError("Invalido Horas Hahu no Horas Remata")
    else:
        if end_date < start_date:
            raise ValueError("Data Remata Labele Uluk husi Data Hahu")
    
        current_date = start_date
        while current_date <= end_date:
            # Exclude weekends
            if current_date.weekday() not in [5, 6]:
                # Calculate number of hours on this day based on start_time_status and end_time_status
                if current_date == start_date:
                    if start_time_status == "08:00":
                        hours = 8.0
                    elif start_time_status == "13:00":
                        hours = 4.0
                    else:
                        raise ValueError("Invalid start_time_status")
                elif current_date == end_date:
                    if end_time_status == "17:00":
                        hours = 8.0
                    elif end_time_status == "12:00":
                        hours = 4.0
                    else:
                        raise ValueError("Invalid end_time_status")
                else:
                    hours = 8.0
                
                total_days += hours / 8.0  # Convert hours to days
            
            current_date += timedelta(days=1)
    
    return total_days


def koko2():
    start_date = datetime(2023, 3, 14).date()
    end_date = datetime(2023, 3, 14).date()
    start_time_status = "08:00"
    end_time_status = "12:00"
    days = calculate_leave_days(start_date, end_date, start_time_status, end_time_status)
    print(days)

   



# Generated by Django 3.2 on 2022-12-20 07:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0037_alter_leavetype_total'),
    ]

    operations = [
        migrations.AlterField(
            model_name='leavetype',
            name='total',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]



import datetime
import numpy as np
from django.shortcuts import render, redirect, get_object_or_404
from django.contrib.auth.decorators import login_required
from settings_app.decorators import allowed_users
from django.contrib import messages

@login_required
def LeaveChartDash(request):
    group = request.user.groups.all()[0].name
    context = {
        'group': group,
        'title': 'Grafiku Licensa', 'legend': 'Grafiku Licensa'
    }
    return render(request, 'leave_chart/chart_leave.html', context)


# Generated by Django 4.0 on 2022-09-25 00:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0015_alter_leavede_obs_alter_leavedelegate_obs_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='leave',
            name='is_print',
            field=models.BooleanField(default=False, null=True),
        ),
    ]



from django.db import models
from django.contrib.auth.models import User
from employee.models import Employee
from settings_app.upload_utils import upload_leave
from django.core.validators import FileExtensionValidator

class Year(models.Model):
	year = models.IntegerField(null=True, blank=True)
	
	def __str__(self):
		template = '{0.year}'
		return template.format(self)

class Month(models.Model):
	code = models.IntegerField(null=True, blank=True)
	name = models.CharField(max_length=20, null=True, blank=True)
	def __str__(self):
		template = '{0.name} '
		return template.format(self)
	
class LeavePeriod(models.Model):
	employee = models.ForeignKey(Employee, on_delete=models.CASCADE, related_name='leaveperiod', null=True)
	start_month = models.ForeignKey(Month, on_delete=models.CASCADE, null=True, blank=True, verbose_name="Fulan Hahu", related_name="start_month")
	start_year = models.ForeignKey(Year, on_delete=models.CASCADE, null=True, blank=True, verbose_name="Tinan Hahu", related_name="start_year")
	start_date = models.DateField(null=True, blank=True, verbose_name="Data Hahu")
	end_date = models.DateField(null=True, blank=True, verbose_name="Data Remate")
	end_month = models.ForeignKey(Month, on_delete=models.CASCADE, null=True, blank=True, verbose_name="Fulan Remata", related_name="end_month")
	end_year = models.ForeignKey(Year, on_delete=models.CASCADE, null=True, blank=True, verbose_name="Tinan Remata", related_name="end_year")
	balance_carry = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True, default=0.00, verbose_name="Balansu Carry Forward") # Balance Cary Forward
	is_active = models.BooleanField(default=False)

	def __str__(self):
		return f'{self.employee} - {self.start_month} - {self.start_year} - {self.end_month} - {self.end_year}'
	

class LeaveType(models.Model):
	name = models.CharField(max_length=20, null=True, blank=True, verbose_name="Naran")
	total = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True)
	start_total = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True, verbose_name="Balance Start")
	def __str__(self):
		template = '{0.name}'
		return template.format(self)

class Leave(models.Model):
	description = models.CharField(max_length=250, null=True, blank=True, verbose_name="Deskrisaun Licensa")
	employee = models.ForeignKey(Employee, on_delete=models.CASCADE, related_name='leave')
	leave_period = models.ForeignKey(LeavePeriod, on_delete=models.CASCADE, null=True, blank=True, verbose_name="Leave Period", related_name="leaveperiod")
	leave_type = models.ForeignKey(LeaveType, on_delete=models.CASCADE, related_name='leave', verbose_name="Tipu Licensa")
	date = models.DateField(null=True, blank=True)
	start_date = models.DateField(null=True, blank=True)
	end_date = models.DateField(null=True, blank=True)
	start_time = models.TimeField(null=True, blank=True, verbose_name="Horas Hahu")
	end_time = models.TimeField(null=True, blank=True, verbose_name="Horas Remata")
	days = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True, verbose_name="Loron")
	start_time_status = models.CharField(choices=[('08:00','08:00'), ('13:00', '13:00')], max_length=15, null=True, blank=True, verbose_name="Horas Hahu Licensa")
	end_time_status = models.CharField(choices=[('12:00','12:00'), ('17:00', '17:00')], max_length=15, null=True, blank=True, verbose_name="Horas Remata Licensa")
	start_date_status = models.CharField(choices=[('Dader','Dader'), ('Lokraik', 'Lokraik')], max_length=15, null=True, blank=True, verbose_name="Horas Data Hahu")
	end_date_status = models.CharField(choices=[('Dader','Dader'), ('Lokraik', 'Lokraik')], max_length=15, null=True, blank=True, verbose_name="Horas Data Remata")
	is_active = models.BooleanField(default=True, null=True)
	is_lock = models.BooleanField(default=False, null=True)
	is_send = models.BooleanField(default=False, null=True)
	is_special = models.BooleanField(default=False, null=True, blank=True)
	is_create_by_hr = models.BooleanField(default=False, null=True, blank=True)
	is_send_to_div = models.BooleanField(default=False, null=True)
	is_delegate = models.BooleanField(default=False, null=True)
	is_update = models.BooleanField(default=False, null=True)
	dep_confirm = models.BooleanField(default=False, null=True)
	dep_send = models.BooleanField(default=False, null=True)
	dep_send_pr = models.BooleanField(default=False, null=True)
	unit_send_pr = models.BooleanField(default=False, null=True)
	unit_confirm = models.BooleanField(default=False, null=True)
	unit_send = models.BooleanField(default=False, null=True)
	vice_send_pr = models.BooleanField(default=False, null=True)
	hr_confirm = models.BooleanField(default=False, null=True)
	hr_send = models.BooleanField(default=False, null=True)
	pr_approve = models.BooleanField(default=False, null=True)
	pr_notify = models.BooleanField(default=False, null=True)
	de_approve = models.BooleanField(default=False, null=True)
	pr_send = models.BooleanField(default=False, null=True)
	is_reject = models.BooleanField(default=False, null=True)
	is_approve = models.BooleanField(default=False, null=True)
	is_finish = models.BooleanField(default=False, null=True)
	is_done = models.BooleanField(default=False, null=True)
	is_print = models.BooleanField(default=False, null=True)
	file = models.FileField(upload_to=upload_leave, null=True, blank=True,
			validators=[FileExtensionValidator(allowed_extensions=['pdf'])], verbose_name="Upload Anekso")
	datetime = models.DateTimeField(null=True)
	user = models.ForeignKey(User, on_delete=models.CASCADE, null=True)
	hashed = models.CharField(max_length=32, null=True)
	def __str__(self):
		template = '{0.employee} - {0.leave_type}'
		return template.format(self)

class LeaveDelegate(models.Model):
	leave = models.OneToOneField(Leave, on_delete=models.CASCADE, related_name='leavedelegate')
	employee = models.ForeignKey(Employee, on_delete=models.CASCADE, null=True, blank=True, related_name='leavedelegate', verbose_name="Delega ba")
	obs = models.CharField(choices=[('Sim','Sim'),('Lae','Lae')], max_length=3, null=True, blank=True, verbose_name="Simu Delegasaun")
	reason = models.TextField(null=True, blank=True, verbose_name="Rajaun")
	is_confirm = models.BooleanField(default=False, null=True)
	is_confirm2 = models.BooleanField(default=False, null=True)
	datetime = models.DateTimeField(null=True)
	user = models.ForeignKey(User, on_delete=models.CASCADE, null=True)
	hashed = models.CharField(max_length=32, null=True)
	def __str__(self):
		template = '{0.leave} - {0.employee}'
		return template.format(self)

class LeaveDep(models.Model):
	leave = models.OneToOneField(Leave, on_delete=models.CASCADE, related_name='leavedep')
	obs = models.CharField(choices=[('Sim','Sim'),('Lae','Lae')], max_length=3, null=True, blank=True, verbose_name="Aprova")
	reason = models.TextField(null=True, blank=True, verbose_name="Rajaun")
	datetime = models.DateTimeField(null=True)
	user = models.ForeignKey(User, on_delete=models.CASCADE, null=True)
	hashed = models.CharField(max_length=32, null=True)
	def __str__(self):
		template = '{0.leave} - {0.obs}'
		return template.format(self)

class LeaveUnit(models.Model):
	leave = models.OneToOneField(Leave, on_delete=models.CASCADE, related_name='leaveunit')
	obs = models.CharField(choices=[('Sim','Sim'),('Lae','Lae')], max_length=3, null=True, blank=True, verbose_name="Aprova")
	reason = models.TextField(null=True, blank=True, verbose_name="Rajaun")
	datetime = models.DateTimeField(null=True)
	user = models.ForeignKey(User, on_delete=models.CASCADE, null=True)
	hashed = models.CharField(max_length=32, null=True)
	def __str__(self):
		template = '{0.leave} - {0.obs}'
		return template.format(self)

class LeaveHR(models.Model):
	leave = models.OneToOneField(Leave, on_delete=models.CASCADE, related_name='leavehr')
	obs = models.CharField(choices=[('Sim','Sim'),('Lae','Lae')], max_length=3, null=True, blank=True, verbose_name="Certifica")
	reason = models.TextField(null=True, blank=True, verbose_name="Rajaun")
	comment = models.TextField(null=True, blank=True, verbose_name="Komentario")
	datetime = models.DateTimeField(null=True)
	user = models.ForeignKey(User, on_delete=models.CASCADE, null=True)
	hashed = models.CharField(max_length=32, null=True)
	def __str__(self):
		template = '{0.leave} - {0.obs}'
		return template.format(self)

class LeaveDE(models.Model):
	leave = models.OneToOneField(Leave, on_delete=models.CASCADE, related_name='leavede')
	obs = models.CharField(choices=[('Sim','Sim'),('Lae','Lae')], max_length=3, null=True, blank=True, verbose_name="Aprova")
	reason = models.TextField(null=True, blank=True, verbose_name="Rajaun")
	datetime = models.DateTimeField(null=True)
	user = models.ForeignKey(User, on_delete=models.CASCADE, null=True)
	hashed = models.CharField(max_length=32, null=True)
	def __str__(self):
		template = '{0.leave} - {0.obs}'
		return template.format(self)

class LeaveCount(models.Model):
	period = models.ForeignKey(LeavePeriod, on_delete=models.CASCADE, null=True, blank=True, related_name="leaveperiodcount")
	employee = models.ForeignKey(Employee, on_delete=models.CASCADE, related_name='leavecount')
	month = models.ForeignKey(Month, on_delete=models.CASCADE, null=True, blank=True, verbose_name="Fulan", related_name="leavecountmonth")
	update_date = models.DateField(null=True, blank=True)
	year = models.ForeignKey(Year, on_delete=models.CASCADE, null=True, blank=True, verbose_name="Tinan", related_name="leavecountyear")
	leave_type = models.ForeignKey(LeaveType, on_delete=models.CASCADE, related_name='leavecount', verbose_name="Tipu Licenca")
	leave_earn = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True, verbose_name="Leave Earn") #Leave Earn 1.67
	total_earn = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True, verbose_name="Total Earn") #Total Earn
	prov_total_earn = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True, verbose_name="Provision Total Earn") #Total Earn
	taken = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True, default=0.00, verbose_name="Taken") #Taken
	total_taken = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True, default=0.00, verbose_name="Total Taken") #Taken
	total = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True, verbose_name="Total") #Total Taken
	balance = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True, verbose_name="Balance") #Balance
	total_balance = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True, verbose_name="Total Balance") 
	last_total_balance = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True, default=0.00, verbose_name="Last Total Balance") 
	balance_month = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True, verbose_name="Balance Month") 
	balance_carry = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True, default=0.00, verbose_name="Balansu Carry Forward") # Balance Cary Forward
	total_balance_leave = models.DecimalField(max_digits=5, decimal_places=2, null=True, blank=True, verbose_name="Total Balance Leave") 
	def __str__(self):
		template = '{0.employee} - {0.leave_type} - {0.month} - {0.year} - {0.total_balance}'
		return template.format(self)



import datetime
from django.shortcuts import render, redirect, get_object_or_404
from django.contrib.auth.decorators import login_required
from settings_app.decorators import allowed_users
from django.contrib import messages
from employee.models import CurEmpDivision
from leave.models import Leave, LeaveCount, LeaveDE
from leave.forms import LeaveDEForm

@login_required
@allowed_users(allowed_roles=['de','deputy'])
def deLeaveApprUpdate(request, hashid):
	leave = get_object_or_404(Leave, hashed=hashid)
	empdiv = CurEmpDivision.objects.get(employee=leave.employee)
	objects = LeaveDE.objects.get(leave=leave)
	if request.method == 'POST':
		form = LeaveDEForm(request.POST, instance=objects)
		if form.is_valid():
			instance = form.save(commit=False)
			instance.datetime = datetime.datetime.now()
			instance.user = request.user
			if instance.obs == 'Sim':
				instance.datetime = datetime.datetime.now()
			instance.save()
			leave.de_approve = True
			leave.save()
			messages.success(request, f'Altera sucessu.')
			return redirect('leave-de-appr-detail', hashid=hashid)
	else: form = LeaveDEForm(instance=objects)
	context = {
		'c_emp': leave.employee, 'empdiv': empdiv, 'leave': leave, 'form': form, 'page': 'de',
		'title': 'Aprovasaun Presidente', 'legend': 'Aprovasaun Presidente'
	}
	return render(request, 'leave/form.html', context)

@login_required
@allowed_users(allowed_roles=['de','deputy'])
def deLeaveApprFinish(request, hashid):
	leave = get_object_or_404(Leave, hashed=hashid)
	leavecount = LeaveCount.objects.filter(employee=leave.employee, leave_type=leave.leave_type).last()
	objects = LeaveDE.objects.get(leave=leave)
	if objects.obs == "Sim":
		leave.is_approve = True
		leave.pr_approve = True
		leave.is_reject = False
	elif objects.obs == "Lae":
		leave.is_reject = True
		leave.is_approve = False
		leave.save()
	leave.is_finish = True
	leave.save()
	messages.success(request, f'Aprova sucessu.')
	return redirect('leave-de-appr-detail', hashid=hashid)


@login_required
@allowed_users(allowed_roles=['de'])
def prLeaveSendNotif(request, hashid):
	leave = get_object_or_404(Leave, hashed=hashid)
	leave.pr_send = True
	leave.is_finish = True
	leave.save()
	messages.success(request, f'Susesu Notifika Aplikante')
	return redirect('leave-de-appr-detail', hashid=hashid)

@login_required
@allowed_users(allowed_roles=['de'])
def prLeaveNotifyHR(request, hashid):
	leave = get_object_or_404(Leave, hashed=hashid)
	leave.pr_notify = True
	leave.save()
	messages.success(request, f'Susesu Notifika RH no Vice Presidente')
	return redirect('leave-de-appr-detail', hashid=hashid)



# Generated by Django 3.2 on 2022-12-10 15:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0026_auto_20221210_2356'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='leave',
            name='end_datetime',
        ),
        migrations.RemoveField(
            model_name='leave',
            name='start_datetime',
        ),
    ]



# Generated by Django 3.2 on 2022-12-17 06:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0033_leave_pr_confirm'),
    ]

    operations = [
        migrations.RenameField(
            model_name='leave',
            old_name='pr_confirm',
            new_name='pr_approve',
        ),
    ]



# Generated by Django 4.0 on 2022-09-23 01:10

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('employee', '0004_employee_emp_id'),
        ('leave', '0002_remove_taskdelegate_date'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='TaskDelegate',
            new_name='LeaveDelegate',
        ),
    ]



# Generated by Django 4.0 on 2022-09-23 13:19

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0011_remove_leave_de_obs_remove_leave_hr_obs_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='leavedelegate',
            name='leave',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='leavedelegate', to='leave.leave'),
        ),
        migrations.AlterField(
            model_name='leaveunit',
            name='leave',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='leaveunit', to='leave.leave'),
        ),
    ]



# Generated by Django 4.0 on 2022-09-23 05:15

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0004_alter_leavedelegate_employee'),
    ]

    operations = [
        migrations.RenameField(
            model_name='leavedelegate',
            old_name='is_approve',
            new_name='is_confirm',
        ),
    ]



# Generated by Django 3.2 on 2023-01-22 06:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0063_leave_is_done'),
    ]

    operations = [
        migrations.AddField(
            model_name='leave',
            name='pr_send',
            field=models.BooleanField(default=False, null=True),
        ),
    ]



# Generated by Django 4.0 on 2022-09-23 13:46

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('leave', '0013_leave_is_finish'),
    ]

    operations = [
        migrations.CreateModel(
            name='LeaveHR',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('obs', models.CharField(blank=True, choices=[('Sim', 'Sim'), ('Lae', 'Lae')], max_length=3, null=True, verbose_name='Observasaun')),
                ('reason', models.TextField(blank=True, null=True, verbose_name='Rajaun')),
                ('datetime', models.DateTimeField(null=True)),
                ('hashed', models.CharField(max_length=32, null=True)),
                ('leave', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='leavehr', to='leave.leave')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='auth.user')),
            ],
        ),
        migrations.CreateModel(
            name='LeaveDE',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('obs', models.CharField(blank=True, choices=[('Sim', 'Sim'), ('Lae', 'Lae')], max_length=3, null=True, verbose_name='Observasaun')),
                ('reason', models.TextField(blank=True, null=True, verbose_name='Rajaun')),
                ('datetime', models.DateTimeField(null=True)),
                ('hashed', models.CharField(max_length=32, null=True)),
                ('leave', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='leavede', to='leave.leave')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='auth.user')),
            ],
        ),
    ]



from django.contrib import admin
from .models import *

admin.site.register(LeaveType)
admin.site.register(Leave)
admin.site.register(LeaveCount)
admin.site.register(LeaveDelegate)
admin.site.register(LeaveUnit)
admin.site.register(LeaveHR)
admin.site.register(LeaveDE)
admin.site.register(LeaveDep)
admin.site.register(Year)
admin.site.register(Month)
admin.site.register(LeavePeriod)


# Generated by Django 3.2 on 2022-12-10 17:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0027_auto_20221211_0022'),
    ]

    operations = [
        migrations.AddField(
            model_name='leave',
            name='end_datetime',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='leave',
            name='start_datetime',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



from django import template

register = template.Library()

@register.filter
def order_by(queryset):
    return queryset.order_by('pk', 'year')

@register.filter
def subs_total_earn(a, b):
    return a - b


import datetime
from django.shortcuts import render, redirect, get_object_or_404
from django.contrib.auth.decorators import login_required
from settings_app.decorators import allowed_users
from django.contrib import messages
from employee.models import CurEmpDivision, Employee
from leave.models import Leave, LeaveCount, LeaveUnit, LeaveDep
from leave.forms import LeaveUnitForm, LeaveDepForm
from settings_app.utils import getnewid
from settings_app.user_utils import c_staff, c_dep

@login_required
@allowed_users(allowed_roles=['unit'])
def cLeaveVerUpdate(request, hashid):
	c_emp = c_staff(request.user)
	empdiv = CurEmpDivision.objects.get(employee=c_emp)
	leave = get_object_or_404(Leave, hashed=hashid)
	objects = LeaveUnit.objects.get(leave=leave)
	if request.method == 'POST':
		form = LeaveUnitForm(request.POST, instance=objects)
		if form.is_valid():
			instance = form.save(commit=False)
			instance.user = request.user
			if instance.obs == 'Sim':
				instance.datetime = datetime.datetime.now()
				leave.is_approve = True
				leave.unit_confirm = True
				leave.save()
			else:
				leave.save()
			instance.save()
			messages.success(request, f'Altera sucessu.')
			return redirect('leave-c-ver-detail', hashid=hashid)
	else: form = LeaveUnitForm(instance=objects)
	context = {
		'c_emp': c_emp, 'empdiv': empdiv, 'leave': leave, 'form': form, 'page': 'unit',
		'title': 'Verfika husi Unidade', 'legend': 'Verfika husi Unidade'
	}
	return render(request, 'leave/form.html', context)

# workd: Leave Dep
@login_required
@allowed_users(allowed_roles=['dep'])
def cLeaveDepUpdate(request, hashid):
	c_emp = c_staff(request.user)
	empdiv = CurEmpDivision.objects.get(employee=c_emp)
	leave = get_object_or_404(Leave, hashed=hashid)
	emp = leave.employee
	objects = LeaveDep.objects.get(leave=leave)
	if request.method == 'POST':
		form = LeaveDepForm(request.POST, instance=objects)
		if form.is_valid():
			instance = form.save(commit=False)
			instance.datetime = datetime.datetime.now()
			instance.user = request.user
			instance.save()
			leave.dep_confirm = True
			leave.save()
			messages.success(request, f'Altera sucessu.')
			return redirect('leave-dep-detail', hashid=leave.leavedep.hashed)
	else: form = LeaveDepForm(instance=objects)
	context = {
		'c_emp': c_emp, 'empdiv': empdiv, 'leave': leave, 'form': form, 'page': 'unit',
		'title': 'Desizaun husi Chefe Ekipa', 'legend': 'Desizaun husi Chefe Ekipa', 'emp': emp
	}
	return render(request, 'leave/form.html', context)
@login_required
@allowed_users(allowed_roles=['dep'])
def cLeaveDepSend(request, hashid):
	c_emp = c_staff(request.user)
	empdiv = CurEmpDivision.objects.get(employee=c_emp)
	leave = get_object_or_404(Leave, hashed=hashid)
	if empdiv.unit.pk == 8 or empdiv.department.unit.pk == 8:
		leave.dep_send_pr = True
		leave.hr_confirm = False
		leave.save()
	else:
		leave.dep_send = True
		leave.hr_confirm = False
		leave.save()
	messages.success(request, f'Susesu Manda Ona')
	return redirect('leave-dep-detail', hashid=leave.leavedep.hashed)


@login_required
@allowed_users(allowed_roles=['unit'])
def cLeaveVerSend(request, hashid):
	c_emp = c_staff(request.user)
	empdiv = CurEmpDivision.objects.get(employee=c_emp)
	leave = get_object_or_404(Leave, hashed=hashid)
	leave.unit_send = True
	leave.is_finish = True
	leave.hr_confirm = False
	leave.save()
	messages.success(request, f'Manda sucessu.')
	return redirect('leave-c-ver-detail', hashid=hashid)

@login_required
@allowed_users(allowed_roles=['dep', 'unit'])
def cLeaveVerReject(request, hashid):
	group = request.user.groups.all()[0].name
	c_emp = c_staff(request.user)
	leave = get_object_or_404(Leave, hashed=hashid)
	leave.is_finish = True
	leave.is_reject = True
	leave.save()
	# leavecount = LeaveCount.objects.filter(employee=leave.employee, leave_type=leave.leave_type).last()
	# leavecount.total = leavecount.total - leave.days
	# leavecount.balance = leavecount.balance + leave.days
	# leave.save()
	messages.success(request, f'Susesu Notifika Aplikante')
	if group == 'dep':
		return redirect('leave-dep-detail', hashid=leave.leavedep.hashid)
	elif group == 'unit':
		return redirect('leave-c-ver-detail', hashid=hashid)



# Generated by Django 3.2 on 2022-12-29 17:21

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0056_auto_20221230_0221'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='leavecount',
            options={},
        ),
    ]



from django.shortcuts import render, get_object_or_404
from django.contrib.auth.decorators import login_required
from contract.models import EmpPosition
from employee.models import CurEmpDivision
from settings_app.decorators import allowed_users
from leave.models import Leave, LeaveCount, LeaveDE, LeaveDelegate, LeaveHR, LeaveUnit

@login_required
@allowed_users(allowed_roles=['admin','hr'])
def hrLeavePrint(request, hashid):
	group = request.user.groups.all()[0].name
	leave = get_object_or_404(Leave, hashed=hashid)
	leavecount = LeaveCount.objects.filter(employee=leave.employee, leave_type=leave.leave_type).first()
	leavedel = LeaveDelegate.objects.filter(leave=leave).first()
	leaveunit = LeaveUnit.objects.filter(leave=leave).first()
	leavehr = LeaveHR.objects.filter(leave=leave).first()
	leavede = LeaveDE.objects.filter(leave=leave).first()
	empdiv = CurEmpDivision.objects.get(employee=leave.employee)
	dep, unit = [],[]
	if empdiv.department:
		dep = EmpPosition.objects.filter(is_active=True, position_id=4, department=empdiv.department).first()
		unit = EmpPosition.objects.filter(is_active=True, position_id=3, unit=empdiv.department.unit).first()
	elif empdiv.unit:
		unit = EmpPosition.objects.filter(is_active=True, position_id=4, unit=empdiv.unit).first()
	de = EmpPosition.objects.filter(is_active=True, position_id=1).first()
	context = {
		'group': group, 'leave': leave, 'leavecount': leavecount, 'leavedel': leavedel, 'leaveunit': leaveunit,
		'leavehr': leavehr, 'leavede': leavede, 'dep': dep, 'unit': unit, 'de': de, 
		'title': 'Imprimi Licensa', 'legend': 'Imprimi Licensa'
	}
	return render(request, 'leave_print/leave_print.html', context)


# Generated by Django 3.2 on 2023-01-14 03:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0060_leavecount_update_date'),
    ]

    operations = [
        migrations.AddField(
            model_name='leavecount',
            name='balance_month',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Balance Month'),
        ),
    ]



# Generated by Django 3.2 on 2022-12-21 01:09

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0048_auto_20221221_1008'),
    ]

    operations = [
        migrations.AddField(
            model_name='leavecount',
            name='month',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='leavecountmonth', to='leave.month', verbose_name='Fulan'),
        ),
        migrations.AddField(
            model_name='leavecount',
            name='year',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='leavecountyear', to='leave.year', verbose_name='Tinan'),
        ),
    ]



# Generated by Django 3.2 on 2022-12-21 01:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0047_auto_20221220_1729'),
    ]

    operations = [
        migrations.AddField(
            model_name='leavecount',
            name='leave_earn',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Leave Earn'),
        ),
        migrations.AddField(
            model_name='leavecount',
            name='total_balance',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Total Balance'),
        ),
        migrations.AddField(
            model_name='leavecount',
            name='total_earn',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Total Earn'),
        ),
        migrations.AlterField(
            model_name='leavecount',
            name='balance',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Balance'),
        ),
        migrations.AlterField(
            model_name='leavecount',
            name='taken',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Taken'),
        ),
        migrations.AlterField(
            model_name='leavecount',
            name='total',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Total Taken'),
        ),
    ]



# Generated by Django 3.2 on 2023-03-13 10:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0070_leave_is_create_by_hr'),
    ]

    operations = [
        migrations.AddField(
            model_name='leave',
            name='end_time_status',
            field=models.CharField(blank=True, choices=[('12:00', '12:00'), ('17:00', '17:00')], max_length=15, null=True, verbose_name='Horas Remata Licensa'),
        ),
        migrations.AddField(
            model_name='leave',
            name='start_time_status',
            field=models.CharField(blank=True, choices=[('08:00', '08:00'), ('13:00', '13:00')], max_length=15, null=True, verbose_name='Horas Hahu Licensa'),
        ),
    ]



from django.db.models.signals import post_save
from django.dispatch import receiver
from django.contrib.auth.models import User
from .models import Leave, LeaveDE, LeaveDelegate, LeaveHR, LeaveUnit, LeaveDep

@receiver(post_save, sender=Leave)
def create_leave(sender, instance, created, **kwargs):
	
	if created:
		LeaveDelegate.objects.create(id=instance.id, leave=instance, hashed=instance.hashed)
		LeaveUnit.objects.create(id=instance.id, leave=instance, hashed=instance.hashed)
		LeaveDep.objects.create(id=instance.id, leave=instance, hashed=instance.hashed)
		LeaveHR.objects.create(id=instance.id, leave=instance, hashed=instance.hashed)
		LeaveDE.objects.create(id=instance.id, leave=instance, hashed=instance.hashed)


# Generated by Django 4.0 on 2022-09-25 04:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0020_remove_leave_s_is_delegate'),
    ]

    operations = [
        migrations.AddField(
            model_name='leave',
            name='is_approve',
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AddField(
            model_name='leave',
            name='is_reject',
            field=models.BooleanField(default=False, null=True),
        ),
    ]



from django import forms
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Submit, Row, Column, Button, HTML
from django.db.models import Q
from django.contrib.auth.models import User
from leave.models import Leave, LeaveDE, LeaveDelegate, LeaveHR, LeaveUnit, LeaveDep, LeavePeriod
from django_summernote.widgets import SummernoteWidget
from django.utils import timezone
import datetime
from django.core.exceptions import ValidationError


class DateInput(forms.DateInput):
	input_type = 'date'

class TimeInput(forms.TimeInput):
    input_type = 'time'

def validate_time_range(value):
    now = datetime.datetime.now().time()
    if now >= datetime.time(18, 0, 0) or now <= datetime.time(6, 0, 0):
        raise ValidationError('Input is blocked between 18:00 and 06:00')

class LeaveForm(forms.ModelForm):

	start_date = forms.DateField(label='Data Hahu', widget=DateInput(), required=False)
	end_date = forms.DateField(label='Data Remata', widget=DateInput(), required=False)
	
	class Meta:
		model = Leave
		fields = [ 'description', 'leave_type','start_date','end_date', 'file', 'start_time_status', 'end_time_status' ]
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.helper = FormHelper()
		self.fields['description'].required = True
		self.fields['start_date'].required = True
		self.fields['end_date'].required = True
		self.fields['start_time_status'].required = True
		self.fields['end_time_status'].required = True
		self.helper.form_method = 'post'
		self.helper.layout = Layout(
			Row(
				Column('description', css_class='form-group col-md-12 mb-0'),
				css_class='form-row'
			),
			Row(
				Column('leave_type', css_class='form-group col-md-6 mb-0'),
                Column('file', css_class='form-group col-md-6 mb-0'),
				css_class='form-row'
			),
			Row(
				Column('start_date', css_class='form-group col-md-3 mb-0'),
                Column('end_date', css_class='form-group col-md-3 mb-0'),
                Column('start_time_status', css_class='form-group col-md-3 mb-0'),
				Column('end_time_status', css_class='form-group col-md-3 mb-0'),
				css_class='form-row'
			),
			HTML(""" <button class="btn btn-primary" type="submit" title="Rai">Rai <i class="fa fa-save"></i></button> """)
		)
class HRLeaveCommentForm(forms.ModelForm):
	comment = forms.CharField(label="Measuring", required=False, widget=SummernoteWidget(attrs={'summernote': {'width': '100%', 'height': '300px'}}))
	
	class Meta:
		model = Leave
		fields = ['comment']
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.helper = FormHelper()
		self.helper.form_method = 'post'
		self.fields['comment'].label = 'Komentario'
		self.fields['comment'].required = True
		self.helper.layout = Layout(
			Row(
				Column('comment', css_class='form-group col-md-12 mb-0'),
				css_class='form-row'
			),
			HTML(""" <button class="btn btn-primary" type="submit" title="Rai">Rai <i class="fa fa-save"></i></button> """)
		)

class HRLeaveForm(forms.ModelForm):
	date = forms.DateField(label='Data Aplika', widget=DateInput(), required=False)
	start_date = forms.DateField(label='Data Hahu', widget=DateInput(), required=False)
	end_date = forms.DateField(label='Data Remata', widget=DateInput(), required=False)
	class Meta:
		model = Leave
		fields = ['description', 'leave_type','date','start_date','end_date', 'days', 'file']
		widgets = {
            "start_date": DateInput(
                attrs={"type": "datetime-local", "class": "form-control", "min": timezone.now().date()},
                format="%Y-%m-%dT%H:%M",
            ),
            "end_date": DateInput(
                attrs={"type": "datetime-local", "class": "form-control"},
                format="%Y-%m-%dT%H:%M",
            ),
        }
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.helper = FormHelper()
		self.helper.form_method = 'post'
		self.fields['description'].required = True
		self.fields['days'].required = True
		self.fields['date'].required = True
		self.fields['start_date'].required = True
		self.fields['end_date'].required = True
		self.helper.layout = Layout(
			Row(
				Column('description', css_class='form-group col-md-12 mb-0'),
				css_class='form-row'
			),
			Row(
				Column('leave_type', css_class='form-group col-md-5 mb-0'),
                Column('date', css_class='form-group col-md-5 mb-0'),
                Column('days', css_class='form-group col-md-2 mb-0'),
				css_class='form-row'
			),
			Row(
				Column('start_date', css_class='form-group col-md-4 mb-0'),
                Column('end_date', css_class='form-group col-md-4 mb-0'),
                Column('file', css_class='form-group col-md-4 mb-0'),
				css_class='form-row'
			),
			HTML(""" <button class="btn btn-primary" type="submit" title="Rai">Rai <i class="fa fa-save"></i></button> """)
		)

class HRLeaveFormSpecial(forms.ModelForm):
	date = forms.DateField(label='Data Aplika', widget=DateInput(), required=False)
	start_date = forms.DateField(label='Data Hahu', widget=DateInput(), required=False)
	end_date = forms.DateField(label='Data Remata', widget=DateInput(), required=False)
	start_time = forms.TimeField(widget=TimeInput)
	end_time = forms.TimeField(widget=TimeInput)
	class Meta:
		model = Leave
		fields = ['description', 'leave_type','date','start_date','end_date', 'file', 'start_time', 'end_time']
		widgets = {
            "start_date": DateInput(
                attrs={"type": "datetime-local", "class": "form-control", "min": timezone.now().date()},
                format="%Y-%m-%dT%H:%M",
            ),
            "end_date": DateInput(
                attrs={"type": "datetime-local", "class": "form-control"},
                format="%Y-%m-%dT%H:%M",
            ),
        }
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.helper = FormHelper()
		self.helper.form_method = 'post'
		self.fields['description'].required = True
		self.fields['date'].required = True
		self.fields['start_date'].required = True
		self.fields['end_date'].required = True
		self.helper.layout = Layout(
			Row(
				Column('description', css_class='form-group col-md-12 mb-0'),
				css_class='form-row'
			),
			Row(
				Column('leave_type', css_class='form-group col-md-6 mb-0'),
                Column('date', css_class='form-group col-md-6 mb-0'),
				css_class='form-row'
			),
			Row(
				Column('start_date', css_class='form-group col-md-3 mb-0'),
                Column('end_date', css_class='form-group col-md-3 mb-0'),
				Column('start_time', css_class='form-group col-md-3 mb-0'),
                Column('end_time', css_class='form-group col-md-3 mb-0'),
				css_class='form-row'
			),
			Row(
                Column('file', css_class='form-group col-md-12 mb-0'),
				css_class='form-row'
			),
			HTML(""" <button class="btn btn-primary" type="submit" title="Rai">Rai <i class="fa fa-save"></i></button> """)
		)

class HRPeriodForm(forms.ModelForm):
	class Meta:
		model = LeavePeriod
		fields = ['start_year','end_year', 'balance_carry']
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.helper = FormHelper()
		self.fields['start_year'].required=True
		self.fields['end_year'].required=True
		self.helper.form_method = 'post'
		self.fields['balance_carry'].label = 'Balance Carry Forward (Opsional)'
		self.helper.layout = Layout(
			Row(
                Column('start_year', css_class='form-group col-md-4 mb-0'),
				Column('end_year', css_class='form-group col-md-4 mb-0'),
				Column('balance_carry', css_class='form-group col-md-4 mb-0'),
				css_class='form-row'
			),
			HTML(""" <button class="btn btn-primary" type="submit" title="Rai">Rai <i class="fa fa-save"></i></button> """)
		)

class LeaveDelegateForm1(forms.ModelForm):
	class Meta:
		model = LeaveDelegate
		fields = ['employee']
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.helper = FormHelper()
		self.helper.form_method = 'post'
		self.helper.layout = Layout(
			Row(
				Column('employee', css_class='form-group col-md-6 mb-0'),
				css_class='form-row'
			),
			HTML(""" <button class="btn btn-primary" type="submit" title="Rai">Rai <i class="fa fa-save"></i></button> """)
		)

class LeaveDelegateForm2(forms.ModelForm):
	class Meta:
		model = LeaveDelegate
		fields = ['obs','reason']
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.helper = FormHelper()
		self.helper.form_method = 'post'
		self.helper.layout = Layout(
			Row(
				Column('obs', css_class='form-group col-md-3 mb-0'),
				Column('reason', css_class='form-group col-md-9 mb-0'),
				css_class='form-row'
			),
			HTML(""" <button class="btn btn-primary" type="submit" title="Rai">Rai <i class="fa fa-save"></i></button> """)
		)

class LeaveUnitForm(forms.ModelForm):
	class Meta:
		model = LeaveUnit
		fields = ['obs','reason']
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.helper = FormHelper()
		self.helper.form_method = 'post'
		self.helper.layout = Layout(
			Row(
				Column('obs', css_class='form-group col-md-3 mb-0'),
				Column('reason', css_class='form-group col-md-9 mb-0'),
				css_class='form-row'
			),
			HTML(""" <button class="btn btn-primary" type="submit" title="Rai">Rai <i class="fa fa-save"></i></button> """)
		)

# work: DEP
class LeaveDepForm(forms.ModelForm):
	class Meta:
		model = LeaveDep
		fields = ['obs','reason']
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.helper = FormHelper()
		self.helper.form_method = 'post'
		self.fields['obs'].required = True
		self.helper.layout = Layout(
			Row(
				Column('obs', css_class='form-group col-md-3 mb-0'),
				Column('reason', css_class='form-group col-md-9 mb-0'),
				css_class='form-row'
			),
			HTML(""" <button class="btn btn-primary" type="submit" title="Rai">Rai <i class="fa fa-save"></i></button> """)
		)

class LeaveHRForm(forms.ModelForm):
	class Meta:
		model = LeaveHR
		fields = ['obs','reason']
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.helper = FormHelper()
		self.helper.form_method = 'post'
		self.helper.layout = Layout(
			Row(
				Column('obs', css_class='form-group col-md-3 mb-0'),
				Column('reason', css_class='form-group col-md-9 mb-0'),
				css_class='form-row'
			),
			HTML(""" <button class="btn btn-primary" type="submit" title="Rai">Rai <i class="fa fa-save"></i></button> """)
		)

class LeaveDEForm(forms.ModelForm):
	class Meta:
		model = LeaveDE
		fields = ['obs','reason']
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.helper = FormHelper()
		self.fields['obs'].required=True
		self.helper.form_method = 'post'
		self.helper.layout = Layout(
			Row(
				Column('obs', css_class='form-group col-md-3 mb-0'),
				Column('reason', css_class='form-group col-md-9 mb-0'),
				css_class='form-row'
			),
			HTML(""" <button class="btn btn-primary" type="submit" title="Rai">Rai <i class="fa fa-save"></i></button> """)
		)

class LeaveFilterForm(forms.Form):
    tinan = forms.ChoiceField(choices=[(0, 'Tinan...')])
    tri = forms.ChoiceField(choices=[(0, 'Trimestral...')])
    fulkan = forms.ChoiceField(choices=[(0, 'Fulan...')])
    unit = forms.ChoiceField(choices=[(0, 'Divizaun...')])


# Generated by Django 4.0 on 2022-10-02 12:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0023_remove_leavedelegate_is_send'),
    ]

    operations = [
        migrations.AlterField(
            model_name='leave',
            name='is_active',
            field=models.BooleanField(default=True, null=True),
        ),
    ]



# Generated by Django 4.0 on 2022-09-25 10:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0022_rename_s_is_finish_leave_is_delegate_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='leavedelegate',
            name='is_send',
        ),
    ]



# Generated by Django 4.0 on 2022-09-21 11:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('employee', '0003_employee_is_new'),
    ]

    operations = [
        migrations.CreateModel(
            name='Leave',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField(blank=True, null=True)),
                ('start_date', models.DateField(blank=True, null=True)),
                ('end_date', models.DateField(blank=True, null=True)),
                ('days', models.IntegerField(blank=True, null=True, verbose_name='Loron')),
                ('is_active', models.BooleanField(default=True)),
                ('is_lock', models.BooleanField(default=False, null=True)),
                ('is_send', models.BooleanField(default=False, null=True)),
                ('dep_confirm', models.BooleanField(default=False, null=True)),
                ('dep_send', models.BooleanField(default=False, null=True)),
                ('unit_confirm', models.BooleanField(default=False, null=True)),
                ('unit_send', models.BooleanField(default=False, null=True)),
                ('de_confirm', models.BooleanField(default=False, null=True)),
                ('datetime', models.DateTimeField(null=True)),
                ('hashed', models.CharField(max_length=32, null=True)),
                ('employee', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='leave', to='employee.employee')),
            ],
        ),
        migrations.CreateModel(
            name='LeaveType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=20, null=True, verbose_name='Naran')),
            ],
        ),
        migrations.CreateModel(
            name='TaskDelegate',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField(blank=True, null=True)),
                ('obs', models.CharField(blank=True, choices=[('Sim', 'Sim'), ('Lae', 'Lae')], max_length=3, null=True, verbose_name='Observasaun')),
                ('reason', models.TextField(blank=True, null=True, verbose_name='Rajaun')),
                ('is_send', models.BooleanField(default=False, null=True)),
                ('is_approve', models.BooleanField(default=False, null=True)),
                ('datetime', models.DateTimeField(null=True)),
                ('hashed', models.CharField(max_length=32, null=True)),
                ('employee', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='leavedelegate', to='employee.employee', verbose_name='Delega ba')),
                ('leave', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='leavedelegate', to='leave.leave')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='auth.user')),
            ],
        ),
        migrations.CreateModel(
            name='LeaveCount',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('total', models.IntegerField(blank=True, null=True)),
                ('balance', models.IntegerField(blank=True, null=True, verbose_name='Balansu')),
                ('datetime', models.DateTimeField(null=True)),
                ('hashed', models.CharField(max_length=32, null=True)),
                ('employee', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='leavecount', to='employee.employee')),
                ('leave_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='leavecount', to='leave.leavetype', verbose_name='Tipu Licenca')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='auth.user')),
            ],
        ),
        migrations.AddField(
            model_name='leave',
            name='leave_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='leave', to='leave.leavetype', verbose_name='Tipu Licensa'),
        ),
        migrations.AddField(
            model_name='leave',
            name='user',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='auth.user'),
        ),
    ]



import datetime
from django.shortcuts import render, redirect, get_object_or_404
from django.contrib.auth.decorators import login_required
from settings_app.decorators import allowed_users
from django.contrib import messages
from employee.models import CurEmpDivision, Employee
from leave.models import Leave, LeaveHR, LeaveUnit, LeaveCount, LeavePeriod, Month as Mnth, Year as Yr, LeaveType, LeaveDep
from leave.forms import LeaveHRForm, LeaveUnitForm, LeaveForm, HRLeaveForm, HRPeriodForm, HRLeaveCommentForm, HRLeaveFormSpecial
from settings_app.utils import getnewid
from settings_app.user_utils import c_staff, c_unit
from dateutil.parser import parser
from datetime import datetime as dt
from datetime import timedelta
from attendance.models import Attendance, AttendanceStatus,Year, Month
import pytz
import pandas as pd
from django.db.models import Q, F
from django.utils import timezone
from calendar import mdays
from contract.models import EmpPosition
from attendance.models import Attendance, AttendanceStatus, Year as YAten, Month as Maten
from leave.utils import createAttendance,createAttendanceEmp, createLeave, count_day,calculate_hours, add_time, createAttendanceSpecial, createLeaveSpecial, createLeaveProcess
from contract.models import Contract

from leave.utils_2 import koko2, calculate_leave_days

@login_required
@allowed_users(allowed_roles=['admin','hr'])
def hrLeaveCertUpdate(request, hashid):
	leave = get_object_or_404(Leave, hashed=hashid)
	empdiv = CurEmpDivision.objects.get(employee=leave.employee)
	objects = LeaveHR.objects.get(leave=leave)
	if request.method == 'POST':
		form = LeaveHRForm(request.POST, instance=objects)
		if form.is_valid():
			instance = form.save(commit=False)
			instance.datetime = timezone.now()
			instance.user = request.user
			instance.save()
			leave.hr_confirm = True
			leave.save()
			messages.success(request, f'Altera sucessu.')
			return redirect('leave-hr-cert-detail', hashid=hashid)
	else: form = LeaveHRForm(instance=objects)
	context = {
		'c_emp': leave.employee, 'empdiv': empdiv, 'leave': leave, 'form': form, 'page': 'hr',
		'title': 'Certifikasaun RH', 'legend': 'Certifikasaun RH'
	}
	return render(request, 'leave/form.html', context)

@login_required
@allowed_users(allowed_roles=['admin','hr'])
def hrLeaveAddComment(request, hashid):
	leave = get_object_or_404(Leave, hashed=hashid)
	empdiv = CurEmpDivision.objects.get(employee=leave.employee)
	objects = LeaveHR.objects.get(leave=leave)
	if request.method == 'POST':
		form = HRLeaveCommentForm(request.POST, instance=objects)
		if form.is_valid():
			instance = form.save(commit=False)
			instance.datetime = timezone.now()
			instance.user = request.user
			instance.save()
			messages.success(request, f'Susesu fo komentario')
			return redirect('leave-hr-app-detail', hashid=hashid)
	else: form = HRLeaveCommentForm(instance=objects)
	context = {
		'c_emp': leave.employee, 'empdiv': empdiv, 'leave': leave, 'form': form, 'page': 'hr',
		'title': 'Fo Komentario', 'legend': 'Fo Komentario'
	}
	return render(request, 'leave/form.html', context)

@login_required
@allowed_users(allowed_roles=['admin','hr'])
def hrLeaveCertSend(request, hashid):
	leave = get_object_or_404(Leave, hashed=hashid)
	leave.hr_send = True
	leave.de_approve = False
	leave.save()
	messages.success(request, f'Manda sucessu.')
	return redirect('leave-hr-cert-detail', hashid=hashid)

@login_required
@allowed_users(allowed_roles=['admin','hr'])
def hrLeaveUpdateEmpRecord(request, hashid):
	employee = get_object_or_404(Employee, hashed=hashid)
	contract = Contract.objects.filter(employee=employee, is_active=True).last()
	if contract:
		leave_type = None
		if employee.sex == 'Male':
			leave_type = LeaveType.objects.exclude(pk=4)
		else:
			leave_type = LeaveType.objects.all()
		today = dt.today().date()
		leave_period = LeavePeriod.objects.filter(is_active=True, employee=employee).last()
		if leave_period:
			check_last_period = LeavePeriod.objects.filter(pk__lt=leave_period.pk, employee=employee).last()
			last_count_period = LeaveCount.objects.filter(employee=employee, period=check_last_period, leave_type_id=1).last()
			last_month_date = today.replace(day=1) - timedelta(days=1)
			min_month = f'{leave_period.start_year.year}-{leave_period.start_month.code}-{leave_period.start_date.day}'
			max_month = f'{leave_period.end_year.year}-{leave_period.end_month.code}'

			months = pd.period_range(min_month, max_month, freq='M')
			i = 0
			while i < len(months):
				leave_check = LeaveCount.objects.filter(employee=employee, period=leave_period).last()
				m = months[i]
				get_m = get_object_or_404(Mnth, code=m.month)
				get_y = get_object_or_404(Yr, year=m.year)
				if leave_check:
					for lt1 in leave_type:
						if lt1.pk == 1:
							lastleaveal = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type_id=1).last()
							earn_per_month = round(float(lt1.total/12),2)
							date_period_string = f"{m.year}-{m.month}-{leave_period.start_date.day}"
							date_period = dt.strptime(date_period_string,"%Y-%m-%d").date()
							lcnew = LeaveCount.objects.create(
								period = leave_period,
								employee=employee,
								leave_type=lt1,
								month = get_m,
								year = get_y,
								update_date=date_period,
								leave_earn = earn_per_month
							)
							if leave_period.balance_carry > 0.00:
								# lcnew.balance = round(float(leave_period.balance_carry) +   float(lcnew.leave_earn) - float(lcnew.taken),2)
								lcnew.balance = round(float(lastleaveal.balance) + float(lcnew.leave_earn) - float(lcnew.taken),2)
								lcnew.total_earn = round( float(leave_period.balance_carry) + float(lastleaveal.prov_total_earn) + float(lcnew.leave_earn),1)
								lcnew.total_balance = round(float(lcnew.total_earn) - float(lcnew.total_taken),1)
								lcnew.balance_carry = lcnew.total_balance
								lcnew.total_balance_leave = float(lt1.total) - float(lcnew.total_balance)
								lcnew.prov_total_earn = float(lastleaveal.prov_total_earn) + float(earn_per_month)
								lcnew.balance_month = round(float(lastleaveal.total_balance) - float(lcnew.taken),2)
								# lcnew.balance_month = round(float(lastleaveal.total_balance_leave) - float(lcnew.taken),2)
								lcnew.save()
							else:
								# lcnew.balance = round(float(lc.leave_earn) - float(lc.taken),2)
								lcnew.balance = round(float(lastleaveal.balance) + float(lcnew.leave_earn) - float(lcnew.taken),2)
								lcnew.total_earn = round(float(lastleaveal.prov_total_earn) + float(lcnew.leave_earn),1)
								lcnew.total_balance = round(float(lcnew.total_earn) - float(lcnew.total_taken),1)
								lcnew.balance_carry = lcnew.total_balance
								lcnew.total_balance_leave = float(lt1.total) - float(lcnew.total_balance)
								lcnew.prov_total_earn = float(lastleaveal.prov_total_earn) + float(earn_per_month)
								lcnew.balance_month = round(float(lastleaveal.total_balance) - float(lcnew.taken),2)
								# lcnew.balance_month = round(float(lastleaveal.total_balance_leave) - float(lcnew.taken),2)
								lcnew.save()
				else:
					for lt in leave_type:
						if lt.pk == 1:
							# work: Check Balance in Last Period
							date_period_string = f"{m.year}-{m.month}-{leave_period.start_date.day}"
							date_period = dt.strptime(date_period_string,"%Y-%m-%d").date()
							if check_last_period:
								earn_per_month = round(float(lt.total/12),2)

								lc = LeaveCount.objects.create(
									period = leave_period,
									employee=employee,
									leave_type=lt,
									month = get_m,
									year = get_y,
									update_date=date_period,
									leave_earn = earn_per_month
								)
								if leave_period.balance_carry > 0.00:
									lc.balance = round(float(leave_period.balance_carry) +   float(lc.leave_earn) - float(lc.taken),2)
									lc.total_earn = round(float(lc.leave_earn) + float(leave_period.total_balance),2)
								else:
									lc.balance = round(float(last_count_period.total_balance) +   float(lc.leave_earn) - float(lc.taken),2)
									lc.total_earn = round(float(lc.leave_earn) + float(last_count_period.total_balance),2)
								# lc.balance = round(float(last_count_period.total_balance) +   float(lc.leave_earn) - float(lc.taken),2)
								# lc.total_earn = round(float(lc.leave_earn) + float(last_count_period.total_balance),2)
								lc.total_balance = round(float(lc.total_earn) - float(lc.total_taken),2)
								lc.balance_carry = lc.total_balance
								lc.total_balance_leave = float(lt.total) - float(lc.total_balance)
								lc.prov_total_earn = lc.total_earn
								lc.balance_month = round(float(lc.total_balance) - float(lc.taken),2)
								lc.save()
							else:
								earn_per_month = round(float(lt.total/12),2)

								lc = LeaveCount.objects.create(
									period = leave_period,
									employee=employee,
									leave_type=lt,
									month = get_m,
									year = get_y,
									update_date=date_period,
									leave_earn = earn_per_month
								)
								if leave_period.balance_carry > 0.00:
									lc.balance = round(float(leave_period.balance_carry) +   float(lc.leave_earn) - float(lc.taken),2)
									lc.total_earn = round(float(leave_period.balance_carry) + float(lc.leave_earn),2)
								else:
									lc.balance = round(float(lc.leave_earn) - float(lc.taken),2)
									lc.total_earn = round(float(lc.leave_earn),2)
								# lc.balance = round(float(lc.leave_earn) - float(lc.taken),2)
								# lc.total_earn = round(float(lc.leave_earn),2)
								lc.total_balance = round(float(lc.balance),2)
								lc.balance_carry = lc.total_balance
								lc.total_balance_leave = float(lt.total) - float(lc.total_balance)
								lc.prov_total_earn = lc.leave_earn
								lc.balance_month = round(float(lc.balance) - float(lc.taken),2)
								lc.save()
						else:
							lcall = LeaveCount.objects.create(
								period = leave_period,
								employee=employee,
								leave_type=lt,
								year = get_y,
							)
							lcall.total_balance = float(lt.total)
							lcall.save()
				
				i+=1
		messages.success(request, f'Susesu Update Record')
		return redirect('leave-hr-leave-record', hashid=hashid)
	else:
		messages.error(request, f'Kontrato ba pessoal refere seidauk iha!!')
		return redirect('leave-hr-leave-record', hashid=hashid)

@login_required
def hrLeaveAdd(request, hashid):
	group = request.user.groups.all()[0].name
	employee = get_object_or_404(Employee, hashed=hashid)
	leave_period = LeavePeriod.objects.filter(employee=employee,is_active=True).last()
	check_last_period = LeavePeriod.objects.filter(employee=employee,  pk__lt=leave_period.pk).last()
	last_count_period = LeaveCount.objects.filter(employee=employee, period=check_last_period, leave_type_id=1).last()
	min_month = f'{leave_period.start_year.year}-{leave_period.start_month.code}'
	max_month = f'{leave_period.end_year.year}-{leave_period.end_month.code}'
	months = pd.period_range(min_month, max_month, freq='M')
	today = dt.today().date()
	c_emp = employee
	emp = employee
	if request.method == 'POST':
		newid, new_hashid = getnewid(Leave)
		form = HRLeaveForm(request.POST, request.FILES)
		if form.is_valid():
			leave_type = form.cleaned_data.get('leave_type')
			days = form.cleaned_data.get('days')
			days = round(float(days),2)
			if leave_type.pk == 1:
				earn_per_month = round(float(leave_type.total/12),2)
			else: earn_per_month = 0.00
			start_date = form.cleaned_data.get('start_date')
			end_date = form.cleaned_data.get('end_date')
			last_two_digits = str(days).split('.')[1][:2]
			last_two_digits = int(last_two_digits)

			min_month = f'{leave_period.start_year.year}-{leave_period.start_month.code}'
			max_month = f'{leave_period.end_year.year}-{leave_period.end_month.code}'
			date1 = pd.to_datetime(start_date, format="%Y %m").date()
			date2 = pd.to_datetime(end_date, format="%Y %m").date()

			start_date_period = pd.to_datetime(min_month, format="%Y %m").date()
			end_date_period = pd.to_datetime(max_month, format="%Y %m").date()
			start_month = date1
			cal_day = count_day(start_date, end_date)
			
			if float(cal_day) >= days:

				# done: CHECK DATA LEAVE IHA PERIODO NIA LARAN
				if start_date_period <= start_month <= end_date_period + datetime.timedelta(days=29):
					current_leave_count = LeaveCount.objects.filter(period=leave_period, employee=employee,leave_type= leave_type, month__code=start_date.month, year__year=start_date.year).last()
					if current_leave_count:
						previous_leave_count = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type= leave_type, month__code__lt=start_date.month, year__year=current_leave_count.year.year).last()
					leave_this_month = LeaveCount.objects.filter(period=leave_period, employee=employee, leave_type_id=1, update_date__lte=today).last()
							
					if leave_this_month.balance >= 0.5:
						if   days <= leave_this_month.balance or days <= leave_this_month.balance + 5 :
							if days < 30  or days >= 30:
								# check_day(days, current_leave_count, previous_leave_count)
								# done: CHECK ANGKA LORON 0.5/1.0/1.5
								if last_two_digits == int(5) or last_two_digits == int(0):
									
									
									
									# done: CHECK LEAVE TYPE BA ANNUAL
									if leave_type.pk == 1:
										# done: CHECK RECORD IHA FULAN IDA NE'E
										# current_leave_count = get_object_or_404(LeaveCount, period=leave_period, employee=employee,leave_type= leave_type, month__code=start_date.month, year__year=start_date.year)
										# previous_leave_count = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type= leave_type, month__code__lt=start_date.month, year__year=current_leave_count.year.year).last()
										# done: CHECK KARIK LEAVE IHA FULAN KLARAN
										if previous_leave_count:
											current_leave_count.taken = round(float(current_leave_count.taken) + days,2)
											current_leave_count.total_taken = round(float(previous_leave_count.taken) + days)
											current_leave_count.balance = round(float(previous_leave_count.balance) + float(current_leave_count.leave_earn) - float(days),2 )
											current_leave_count.total_balance = round(float(current_leave_count.total_earn) - float(current_leave_count.total_taken),1)
											current_leave_count.balance_month = round(float(previous_leave_count.balance) - float(current_leave_count.taken),2)
											current_leave_count.save()


											#done: KRIA ATTENDANCE
											createAttendance(request, leave_type, Attendance, AttendanceStatus, employee, start_date, end_date, YAten, Maten)							
										# done: CHECK KARIK LEAVE IHA FULAN PRIMEIRU
										else:
											current_leave_count.taken = round(float(current_leave_count.taken) + days,2)
											if check_last_period:
												current_leave_count.total_taken = current_leave_count.taken
												current_leave_count.total_earn = round(float(last_count_period.total_balance) + float(earn_per_month),1)
												current_leave_count.balance = round(float(last_count_period.total_balance) + float(earn_per_month) - float(current_leave_count.taken),2 )
												current_leave_count.total_balance = round(float(current_leave_count.total_earn) - float(current_leave_count.total_taken),1)
												current_leave_count.balance_month = round(float(current_leave_count.total_balance) - float(current_leave_count.taken),2)
												current_leave_count.save()
											else:
												current_leave_count.total_taken = current_leave_count.taken
												current_leave_count.balance = round(float(current_leave_count.leave_earn) - float(current_leave_count.taken),2 )
												current_leave_count.total_balance = round(float(current_leave_count.total_earn) - float(current_leave_count.total_taken),1)
												current_leave_count.balance_month = round(float(current_leave_count.balance) - float(current_leave_count.taken),2)
												current_leave_count.save()

											# done: KRIA ATTENDANCE
											createAttendance(request, leave_type, Attendance, AttendanceStatus, employee, start_date, end_date, YAten, Maten)


										i = 0
										# done: UPDATE LEAVE RECORD
										while i < len(months):
											m = months[i]
											if i == int(0):
												# work: First Month
												if check_last_period:
													first_leave_count = LeaveCount.objects.filter(employee=employee,month__code=m.month, year__year=m.year, period=leave_period,leave_type=leave_type).first()
													first_leave_count.balance = round(float(last_count_period.total_balance) + float(earn_per_month) -  float(first_leave_count.taken),2)
													first_leave_count.total_earn = round(float(first_leave_count.leave_earn) + float(last_count_period.total_balance),2)
													first_leave_count.total_balance = round(float(first_leave_count.total_earn) - float(first_leave_count.total_taken),2)
													first_leave_count.balance_carry = first_leave_count.total_balance
													first_leave_count.total_balance_leave = float(leave_type.total) - float(first_leave_count.total_balance)
													first_leave_count.prov_total_earn = first_leave_count.total_earn
													first_leave_count.balance_month = round(float(first_leave_count.balance) - float(first_leave_count.taken),2)
													first_leave_count.save()
												else:
													if leave_period.balance_carry > 0.00:
														first_leave_count = LeaveCount.objects.filter(employee=employee,month__code=m.month, year__year=m.year, period=leave_period,leave_type=leave_type).first()
														first_leave_count.balance = round(float(leave_period.balance_carry) + float(first_leave_count.leave_earn) - float(first_leave_count.taken),2)
														first_leave_count.total_earn = round(float(leave_period.balance_carry) + float(first_leave_count.leave_earn),2)
														first_leave_count.total_balance = round( float(first_leave_count.balance),2)
														first_leave_count.balance_carry = first_leave_count.total_balance
														first_leave_count.total_balance_leave = float(leave_period.balance_carry) + float(leave_type.total) - float(first_leave_count.total_balance)
														first_leave_count.prov_total_earn =  float(leave_period.balance_carry) + float(first_leave_count.leave_earn)
														first_leave_count.balance_month = round(float(first_leave_count.balance) - float(first_leave_count.taken),2)
														first_leave_count.save()
													else:
														first_leave_count = LeaveCount.objects.filter(employee=employee,month__code=m.month, year__year=m.year, period=leave_period,leave_type=leave_type).first()
														first_leave_count.balance = round(float(first_leave_count.leave_earn) - float(first_leave_count.taken),2)
														first_leave_count.total_earn = round(float(first_leave_count.leave_earn),2)
														first_leave_count.total_balance = round(float(first_leave_count.balance),2)
														first_leave_count.balance_carry = first_leave_count.total_balance
														first_leave_count.total_balance_leave = float(leave_type.total) - float(first_leave_count.total_balance)
														first_leave_count.prov_total_earn = first_leave_count.leave_earn
														first_leave_count.balance_month = round(float(first_leave_count.balance) - float(first_leave_count.taken),2)
														first_leave_count.save()

											else:
												prev_mont = m.asfreq("M", "S") - 1
												lccurrent = get_object_or_404(LeaveCount, employee=employee, period=leave_period, leave_type=leave_type,  year__year=m.year, month__code=m.month)
												lclast = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type=leave_type,  year__year=prev_mont.year, month__code=prev_mont.month).first()

												if leave_period.balance_carry > 0.00:
													lccurrent.balance = round(float(lclast.balance) + float(lccurrent.leave_earn) - float(lccurrent.taken),2)
													lccurrent.total_earn = round(float(lclast.prov_total_earn) + float(lccurrent.leave_earn),1)
													lccurrent.total_taken = round(float(lclast.total_taken) + float(lccurrent.taken),2)
													lccurrent.total_balance = round(float(lccurrent.total_earn) - float(lccurrent.total_taken),1)
													lccurrent.balance_carry = lccurrent.total_balance
													lccurrent.total_balance_leave = float(leave_type.total) - float(lccurrent.total_balance)
													lccurrent.prov_total_earn = float(lclast.prov_total_earn) + float(earn_per_month)
													lccurrent.balance_month = round(float(lclast.balance) - float(lccurrent.taken),2)
													lccurrent.save()
												else:
													lccurrent.balance = round(float(lclast.balance) + float(lccurrent.leave_earn) - float(lccurrent.taken),2)
													lccurrent.total_earn = round(float(lclast.prov_total_earn) + float(lccurrent.leave_earn),1)
													lccurrent.total_taken = round(float(lclast.total_taken) + float(lccurrent.taken),2)
													lccurrent.total_balance = round(float(lccurrent.total_earn) - float(lccurrent.total_taken),1)
													lccurrent.balance_carry = lccurrent.total_balance
													lccurrent.total_balance_leave = float(leave_type.total) - float(lccurrent.total_balance)
													lccurrent.prov_total_earn = float(lclast.prov_total_earn) + float(earn_per_month)
													lccurrent.balance_month = round(float(lclast.balance) - float(lccurrent.taken),2)
													lccurrent.save()

											i+=1

										# done: KRIA LEAVE
										instance = form.save(commit=False)
										createLeave(request, instance,newid,new_hashid, employee, leave_period)
										messages.success(request, 'Leave Aumenta ho Susesu')
										return redirect('leave-hr-leave-record', c_emp.hashed)



									# done: CHECK KARIK LEAVE LAOS ANNUAL LEAVE
									else:

										lcount = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type=leave_type, month__isnull=False)
										getmonth = get_object_or_404(Mnth, code=start_date.month)
										getyear = get_object_or_404(Yr, year=start_date.year)

										# done: CHECK LEAVE COUNT NEBE IHA ONA MAIBE FULAN LAIHA
										if lcount:
											previous_leave_count = LeaveCount.objects.filter(period=leave_period, employee=employee,leave_type=leave_type).last()
											month_leave_count = LeaveCount.objects.filter(period=leave_period, employee=employee,leave_type= leave_type, month__code=start_date.month, year__year=start_date.year).last()
											# work: Leave Nebe Iha ona
											if days <= previous_leave_count.total_balance:
												if month_leave_count:
													month_leave_count.taken = round(float(month_leave_count.taken) + days, 2)
													month_leave_count.total_taken = round(float(previous_leave_count.total_taken) +  days,2)
													# month_leave_count.total_taken = month_leave_count.taken
													month_leave_count.total_balance = round(float(leave_type.total) - float(month_leave_count.total_taken),2)
													month_leave_count.save()
													instance = form.save(commit=False)
													createLeave(request, instance,newid,new_hashid, employee, leave_period)
													createAttendance(request, leave_type, Attendance, AttendanceStatus, employee, start_date, end_date, YAten, Maten)
													messages.success(request, 'Leave Aumenta ho Susesu')
													return redirect('leave-hr-leave-record', c_emp.hashed)
												else:
													lc = LeaveCount.objects.create(
														period = leave_period,
														employee = employee,
														leave_type = leave_type,
														year = getyear,
														month = getmonth

													)
													lc.taken = days
													lc.total_taken = round(float(previous_leave_count.total_taken) + float(lc.taken),2)
													lc.total_balance = round(float(leave_type.total) - float(lc.total_taken),2)
													lc.save()
													instance = form.save(commit=False)

													createLeave(request, instance,newid,new_hashid, employee, leave_period)
													createAttendance(request, leave_type, Attendance, AttendanceStatus, employee, start_date, end_date, YAten, Maten)

													messages.success(request, 'Leave Aumenta ho Susesu')
													return redirect('leave-hr-leave-record', c_emp.hashed)
											else:
												messages.error(request, 'Loron nebe hili barak liu. Halo favor altera loron nebe ita prienche')
										# done: CHECK LEAVE COUNT NEBE IHA NO FULAN ONA IHA
										else:
											lcount2 = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type=leave_type)
											lcount2.update(
												month=getmonth,
												year=getyear,
												taken = days,
												total_taken = days,
												total_balance = round(float(leave_type.total) - float(days),2)
											)
											instance = form.save(commit=False)
											createLeave(request, instance,newid,new_hashid, employee, leave_period)

											# KRIA ATTENDANCE
											createAttendance(request, leave_type, Attendance, AttendanceStatus, employee, start_date, end_date, YAten, Maten)

											messages.success(request, 'Leave Aumenta ho Susesu')
											return redirect('leave-hr-leave-record', c_emp.hashed)
									
									
									
								else:
									messages.error(request,'Total loron nebe ita prienche latuir formato!!')
							else:
								messages.error(request, f'Loron nebe hili barak liu. Halo favor hare regulamentu konaba total licenca.')
						else:
							messages.error(request, f'Loron nebe hili barak liu. Halo favor hare regulamentu konaba total licenca.')
					else:
						messages.error(request, f'Loron nebe hili barak liu. Halo favor hare regulamentu konaba total licenca.')
				
				
				# done: ERROR KARIK DATA HAHU LAIHA PERIODE NIA LARAN
				else:
					messages.error(request,'Data hahu laiha periode ida ne nia laran. Halo Favor Kria Uluk lai Periode Foun!!')

			else:
				messages.error(request,"Total Loron ne'ebe ita prienche iha Data Hahu no Remata Lahanesan ho Total Loron ne'ebe ita Input")

	else:
		form = HRLeaveForm()
	context = {
		'group': group, 'c_emp': c_emp,  'page': 'record',
		'form': form, 'emp':emp,
		'title': 'Aplika Licensa', 'legend': 'Aplika Licensa'
	}
	return render(request, 'leave/form.html', context)


@login_required
def hrLeaveAddSpecial(request, hashid):
	group = request.user.groups.all()[0].name
	employee = get_object_or_404(Employee, hashed=hashid)
	leave_period = LeavePeriod.objects.filter(employee=employee,is_active=True).last()
	check_last_period = LeavePeriod.objects.filter(employee=employee,  pk__lt=leave_period.pk).last()
	last_count_period = LeaveCount.objects.filter(employee=employee, period=check_last_period, leave_type_id=1).last()
	min_month = f'{leave_period.start_year.year}-{leave_period.start_month.code}'
	max_month = f'{leave_period.end_year.year}-{leave_period.end_month.code}'
	months = pd.period_range(min_month, max_month, freq='M')
	today = dt.today().date()
	c_emp = employee
	emp = employee
	if request.method == 'POST':
		newid, new_hashid = getnewid(Leave)
		form = HRLeaveFormSpecial(request.POST, request.FILES)
		if form.is_valid():
			leave_type = form.cleaned_data.get('leave_type')
			start_time = form.cleaned_data.get('start_time')
			start_time_str = start_time.strftime("%H:%M")
			end_time = form.cleaned_data.get('end_time')
			end_time_str = end_time.strftime("%H:%M")
			tot_hours = calculate_hours(start_time_str, end_time_str)

			days = tot_hours
			# days = round(float(days),2)
			if leave_type.pk == 1:
				earn_per_month = round(float(leave_type.total/12),2)
			else: earn_per_month = 0.00
			start_date = form.cleaned_data.get('start_date')
			end_date = form.cleaned_data.get('end_date')

			min_month = f'{leave_period.start_year.year}-{leave_period.start_month.code}'
			max_month = f'{leave_period.end_year.year}-{leave_period.end_month.code}'
			date1 = pd.to_datetime(start_date, format="%Y %m").date()
			date2 = pd.to_datetime(end_date, format="%Y %m").date()

			start_date_period = pd.to_datetime(min_month, format="%Y %m").date()
			end_date_period = pd.to_datetime(max_month, format="%Y %m").date()
			start_month = date1
			date_range = pd.date_range(start=start_date, end=end_date, freq="B")
			total_length = len(date_range)
			total_day = round(days * total_length,2)


			# done: CHECK DATA LEAVE IHA PERIODO NIA LARAN
			if start_date_period <= start_month <= end_date_period + datetime.timedelta(days=29):
				current_leave_count = LeaveCount.objects.filter(period=leave_period, employee=employee,leave_type= leave_type, month__code=start_date.month, year__year=start_date.year).last()
				if current_leave_count:
					previous_leave_count = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type= leave_type, month__code__lt=start_date.month, year__year=current_leave_count.year.year).last()
				leave_this_month = LeaveCount.objects.filter(period=leave_period, employee=employee, leave_type_id=1, update_date__lte=today).last()

				if leave_this_month.balance >= 0.5:
					if   total_day <= leave_this_month.balance or total_day <= leave_this_month.balance + 5 :
						if total_day < 30  or total_day >= 30:
							# check_day(days, current_leave_count, previous_leave_count)
							# done: CHECK ANGKA LORON 0.5/1.0/1.5
								
								
							# done: CHECK LEAVE TYPE BA ANNUAL
							if leave_type.pk == 1:
								# done: CHECK RECORD IHA FULAN IDA NE'E
								current_leave_count = get_object_or_404(LeaveCount, period=leave_period, employee=employee,leave_type= leave_type, month__code=start_date.month, year__year=start_date.year)
								previous_leave_count = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type= leave_type, month__code__lt=start_date.month, year__year=current_leave_count.year.year).last()
								# done: CHECK KARIK LEAVE IHA FULAN KLARAN
								if previous_leave_count:
									current_leave_count.taken = round(float(current_leave_count.taken) + total_day,2)
									current_leave_count.total_taken = round(float(previous_leave_count.taken) + total_day)
									current_leave_count.balance = round(float(previous_leave_count.balance) + float(current_leave_count.leave_earn) - float(total_day),2 )
									current_leave_count.total_balance = round(float(current_leave_count.total_earn) - float(current_leave_count.total_taken),1)
									current_leave_count.balance_month = round(float(previous_leave_count.balance) - float(current_leave_count.taken),2)
									current_leave_count.save()


									#done: KRIA ATTENDANCE
									createAttendanceSpecial(request, leave_type, Attendance, AttendanceStatus, employee, start_date, end_date, YAten, Maten,start_time_str, end_time_str)							
								# done: CHECK KARIK LEAVE IHA FULAN PRIMEIRU
								else:
									current_leave_count.taken = round(float(current_leave_count.taken) + total_day,2)
									if check_last_period:
										current_leave_count.total_taken = current_leave_count.taken
										current_leave_count.total_earn = round(float(last_count_period.total_balance) + float(earn_per_month),1)
										current_leave_count.balance = round(float(last_count_period.total_balance) + float(earn_per_month) - float(current_leave_count.taken),2 )
										current_leave_count.total_balance = round(float(current_leave_count.total_earn) - float(current_leave_count.total_taken),1)
										current_leave_count.balance_month = round(float(current_leave_count.total_balance) - float(current_leave_count.taken),2)
										current_leave_count.save()
									else:
										current_leave_count.total_taken = current_leave_count.taken
										current_leave_count.balance = round(float(current_leave_count.leave_earn) - float(current_leave_count.taken),2 )
										current_leave_count.total_balance = round(float(current_leave_count.total_earn) - float(current_leave_count.total_taken),1)
										current_leave_count.balance_month = round(float(current_leave_count.balance) - float(current_leave_count.taken),2)
										current_leave_count.save()

									# done: KRIA ATTENDANCE
									createAttendanceSpecial(request, leave_type, Attendance, AttendanceStatus, employee, start_date, end_date, YAten, Maten, start_time_str, end_time_str)


								i = 0
								# done: UPDATE LEAVE RECORD
								while i < len(months):
									m = months[i]
									if i == int(0):
										# work: First Month
										if check_last_period:
											first_leave_count = LeaveCount.objects.filter(employee=employee,month__code=m.month, year__year=m.year, period=leave_period,leave_type=leave_type).first()
											first_leave_count.balance = round(float(last_count_period.total_balance) + float(earn_per_month) -  float(first_leave_count.taken),2)
											first_leave_count.total_earn = round(float(first_leave_count.leave_earn) + float(last_count_period.total_balance),2)
											first_leave_count.total_balance = round(float(first_leave_count.total_earn) - float(first_leave_count.total_taken),2)
											first_leave_count.balance_carry = first_leave_count.total_balance
											first_leave_count.total_balance_leave = float(leave_type.total) - float(first_leave_count.total_balance)
											first_leave_count.prov_total_earn = first_leave_count.total_earn
											first_leave_count.balance_month = round(float(first_leave_count.balance) - float(first_leave_count.taken),2)
											first_leave_count.save()
										else:
											if leave_period.balance_carry > 0.00:
												first_leave_count = LeaveCount.objects.filter(employee=employee,month__code=m.month, year__year=m.year, period=leave_period,leave_type=leave_type).first()
												first_leave_count.balance = round(float(leave_period.balance_carry) + float(first_leave_count.leave_earn) - float(first_leave_count.taken),2)
												first_leave_count.total_earn = round(float(leave_period.balance_carry) + float(first_leave_count.leave_earn),2)
												first_leave_count.total_balance = round( float(first_leave_count.balance),2)
												first_leave_count.balance_carry = first_leave_count.total_balance
												first_leave_count.total_balance_leave = float(leave_period.balance_carry) + float(leave_type.total) - float(first_leave_count.total_balance)
												first_leave_count.prov_total_earn =  float(leave_period.balance_carry) + float(first_leave_count.leave_earn)
												first_leave_count.balance_month = round(float(first_leave_count.balance) - float(first_leave_count.taken),2)
												first_leave_count.save()
											else:
												first_leave_count = LeaveCount.objects.filter(employee=employee,month__code=m.month, year__year=m.year, period=leave_period,leave_type=leave_type).first()
												first_leave_count.balance = round(float(first_leave_count.leave_earn) - float(first_leave_count.taken),2)
												first_leave_count.total_earn = round(float(first_leave_count.leave_earn),2)
												first_leave_count.total_balance = round(float(first_leave_count.balance),2)
												first_leave_count.balance_carry = first_leave_count.total_balance
												first_leave_count.total_balance_leave = float(leave_type.total) - float(first_leave_count.total_balance)
												first_leave_count.prov_total_earn = first_leave_count.leave_earn
												first_leave_count.balance_month = round(float(first_leave_count.balance) - float(first_leave_count.taken),2)
												first_leave_count.save()

									else:
										prev_mont = m.asfreq("M", "S") - 1
										lccurrent = get_object_or_404(LeaveCount, employee=employee, period=leave_period, leave_type=leave_type,  year__year=m.year, month__code=m.month)
										lclast = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type=leave_type,  year__year=prev_mont.year, month__code=prev_mont.month).first()

										if leave_period.balance_carry > 0.00:
											lccurrent.balance = round(float(lclast.balance) + float(lccurrent.leave_earn) - float(lccurrent.taken),2)
											lccurrent.total_earn = round(float(lclast.prov_total_earn) + float(lccurrent.leave_earn),1)
											lccurrent.total_taken = round(float(lclast.total_taken) + float(lccurrent.taken),2)
											lccurrent.total_balance = round(float(lccurrent.total_earn) - float(lccurrent.total_taken),1)
											lccurrent.balance_carry = lccurrent.total_balance
											lccurrent.total_balance_leave = float(leave_type.total) - float(lccurrent.total_balance)
											lccurrent.prov_total_earn = float(lclast.prov_total_earn) + float(earn_per_month)
											lccurrent.balance_month = round(float(lclast.balance) - float(lccurrent.taken),2)
											lccurrent.save()
										else:
											lccurrent.balance = round(float(lclast.balance) + float(lccurrent.leave_earn) - float(lccurrent.taken),2)
											lccurrent.total_earn = round(float(lclast.prov_total_earn) + float(lccurrent.leave_earn),1)
											lccurrent.total_taken = round(float(lclast.total_taken) + float(lccurrent.taken),2)
											lccurrent.total_balance = round(float(lccurrent.total_earn) - float(lccurrent.total_taken),1)
											lccurrent.balance_carry = lccurrent.total_balance
											lccurrent.total_balance_leave = float(leave_type.total) - float(lccurrent.total_balance)
											lccurrent.prov_total_earn = float(lclast.prov_total_earn) + float(earn_per_month)
											lccurrent.balance_month = round(float(lclast.balance) - float(lccurrent.taken),2)
											lccurrent.save()

									i+=1

								# done: KRIA LEAVE
								instance = form.save(commit=False)
								createLeaveSpecial(request, instance,newid,new_hashid, employee, leave_period, total_day)
								messages.success(request, 'Leave Aumenta ho Susesu')
								return redirect('leave-hr-leave-record', c_emp.hashed)



							# done: CHECK KARIK LEAVE LAOS ANNUAL LEAVE
							else:

								lcount = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type=leave_type, month__isnull=False)
								getmonth = get_object_or_404(Mnth, code=start_date.month)
								getyear = get_object_or_404(Yr, year=start_date.year)

								# done: CHECK LEAVE COUNT NEBE IHA ONA MAIBE FULAN LAIHA
								if lcount:
									previous_leave_count = LeaveCount.objects.filter(period=leave_period, employee=employee,leave_type=leave_type).last()
									month_leave_count = LeaveCount.objects.filter(period=leave_period, employee=employee,leave_type= leave_type, month__code=start_date.month, year__year=start_date.year).last()
									# work: Leave Nebe Iha ona
									if days <= previous_leave_count.total_balance:
										if month_leave_count:
											month_leave_count.taken = round(float(month_leave_count.taken) + total_day, 2)
											month_leave_count.total_taken = round(float(previous_leave_count.total_taken) +  total_day,2)
											month_leave_count.total_taken = month_leave_count.taken
											month_leave_count.total_balance = round(float(leave_type.total) - float(month_leave_count.total_taken),2)
											month_leave_count.save()
											instance = form.save(commit=False)
											createLeaveSpecial(request, instance,newid,new_hashid, employee, leave_period, total_day)
											createAttendanceSpecial(request, leave_type, Attendance, AttendanceStatus, employee, start_date, end_date, YAten, Maten, start_time_str, end_time_str)
											messages.success(request, 'Leave Aumenta ho Susesu')
											return redirect('leave-hr-leave-record', c_emp.hashed)
										else:
											lc = LeaveCount.objects.create(
												period = leave_period,
												employee = employee,
												leave_type = leave_type,
												year = getyear,
												month = getmonth

											)
											lc.taken = total_day
											lc.total_taken = round(float(previous_leave_count.total_taken) + float(lc.taken),2)
											lc.total_balance = round(float(leave_type.total) - float(lc.total_taken),2)
											lc.save()
											instance = form.save(commit=False)

											createLeaveSpecial(request, instance,newid,new_hashid, employee, leave_period, total_day)
											createAttendanceSpecial(request, leave_type, Attendance, AttendanceStatus, employee, start_date, end_date, YAten, Maten, start_time_str, end_time_str)

											messages.success(request, 'Leave Aumenta ho Susesu')
											return redirect('leave-hr-leave-record', c_emp.hashed)
									else:
										messages.error(request, 'Loron nebe hili barak liu. Halo favor altera loron nebe ita prienche')
								# done: CHECK LEAVE COUNT NEBE IHA NO FULAN ONA IHA
								else:
									lcount2 = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type=leave_type)
									lcount2.update(
										month=getmonth,
										year=getyear,
										taken = total_day,
										total_taken = total_day,
										total_balance = round(float(leave_type.total) - float(total_day),2)
									)
									instance = form.save(commit=False)
									createLeaveSpecial(request, instance,newid,new_hashid, employee, leave_period, total_day)

									# KRIA ATTENDANCE
									createAttendanceSpecial(request, leave_type, Attendance, AttendanceStatus, employee, start_date, end_date, YAten, Maten, start_time_str, end_time_str)

									messages.success(request, 'Leave Aumenta ho Susesu')
									return redirect('leave-hr-leave-record', c_emp.hashed)
								
								
						else:
							messages.error(request, f'Loron nebe hili barak liu. Halo favor hare regulamentu konaba total licenca.')
					else:
						messages.error(request, f'Loron nebe hili barak liu. Halo favor hare regulamentu konaba total licenca.')
				else:
					messages.error(request, f'Loron nebe hili barak liu. Halo favor hare regulamentu konaba total licenca.')
			# done: ERROR KARIK DATA HAHU LAIHA PERIODE NIA LARAN
			else:
				messages.error(request,'Data hahu laiha periode ida ne nia laran. Halo Favor Kria Uluk lai Periode Foun!!')

			
	else:
		form = HRLeaveFormSpecial()
	context = {
		'group': group, 'c_emp': c_emp,  'page': 'record',
		'form': form, 'emp':emp,
		'title': 'Aplika Licensa', 'legend': 'Aplika Licensa'
	}
	return render(request, 'leave/form.html', context)


@login_required
def hrLeaveAddProcess(request, hashid):
	c_emp = get_object_or_404(Employee, hashed=hashid)
	group = c_emp.employeeuser.user.groups.all()[0].name
	lp = LeavePeriod.objects.filter(employee=c_emp,is_active=True).exists()
	period = LeavePeriod.objects.filter(employee=c_emp,is_active=True).last()
	
	if period :

		empdiv = CurEmpDivision.objects.get(employee=c_emp)
		if lp == True:
			if request.method == 'POST':
				newid, new_hashid = getnewid(Leave)
				form = LeaveForm(request.POST, request.FILES)
				if form.is_valid():
					today = dt.today().date()
					leave_period = LeavePeriod.objects.filter(employee=c_emp,is_active=True).last()
					start_date = form.cleaned_data.get('start_date')
					end_date = form.cleaned_data.get('end_date')
					start_time_status = form.cleaned_data.get('start_time_status')
					end_time_status = form.cleaned_data.get('end_time_status')

					start_dt = dt(start_date.year, start_date.month, start_date.day)
					end_dt = dt(end_date.year, end_date.month, end_date.day)
					try:
						days = calculate_leave_days(start_dt, end_dt, start_time_status, end_time_status)
					except ValueError as e:
						messages.warning(request, str(e))
						return render(request, 'leave/form.html', 
		    				{ 
							'message':e, 
							'group': group, 'c_emp': c_emp, 'empdiv': empdiv, 'form': form, 'page': 'apply',
							'title': 'Formulario Aplika Licensa', 'legend': 'Formulario Aplika Licensa'
							}
							)
					days = round(float(days),2)
					last_two_digits = str(days).split('.')[1][:2]
					last_two_digits = int(last_two_digits)

					min_month = f'{leave_period.start_year.year}-{leave_period.start_month.code}'
					max_month = f'{leave_period.end_year.year}-{leave_period.end_month.code}'
					date1 = pd.to_datetime(start_date, format="%Y %m").date()
					date2 = pd.to_datetime(end_date, format="%Y %m").date()

					start_date_period = pd.to_datetime(min_month, format="%Y %m").date()
					end_date_period = pd.to_datetime(max_month, format="%Y %m").date()
					start_month = date1
					# cal_day = count_day(start_date, end_date)

					if start_date_period <= start_month <= end_date_period + datetime.timedelta(days=29):
						
						leave_this_month = LeaveCount.objects.filter(period=leave_period, employee=c_emp, leave_type_id=1, update_date__lte=today).last()
						if leave_this_month.balance >= 0.5:
							if   days <= leave_this_month.balance or days <= leave_this_month.balance + 5 :

								if days < 30  or days >= 30:
									
									if last_two_digits == int(5) or last_two_digits == int(0):
										instance = form.save(commit=False)
										instance.id = newid
										instance.employee = c_emp
										if group == "unit":
											instance.unit_confirm = True
											instance.unit_send = True
										if group == "dep":
											instance.is_lock = True
											instance.is_send = True
										if group == "de":
											instance.is_lock = True
											instance.is_approve = True
											instance.is_finish = True
											instance.is_finish = True
											instance.pr_send = True
											instance.obs = "Sim"
											instance.pr_approve = True
										instance.datetime = datetime.datetime.now()
										instance.user = request.user
										instance.hashed = new_hashid
										instance.leave_period  = period
										instance.is_create_by_hr  = True
										instance.days = days
										instance.save()
										if group == "unit":
											leaveunit = LeaveUnit.objects.filter(leave_id=newid).first()
											leaveunit.obs = "Sim"
											leaveunit.user = request.user
											leaveunit.datetime = datetime.datetime.now()
											leaveunit.save()
										if group == "dep":

											leavedep = LeaveDep.objects.filter(leave_id=newid).first()
											leavedep.obs = "Sim"
											leavedep.user = request.user
											leavedep.datetime = datetime.datetime.now()
											leavedep.save()
										
										messages.success(request, f'Aumeta sucessu.')
										return redirect('leave-hr-leave-record', c_emp.hashed)
									else:
										messages.error(request,'Total loron nebe ita prienche latuir formato!! Total Loron Nebe Valido: *.00 Sura ba loron  no  *.5 Sura ba "Stengah Hari" ')
								else:
									messages.error(request, f'Loron nebe hili barak liu. Halo favor hare regulamentu konaba total licenca.')
							else:
								messages.error(request, f'Loron nebe hili barak liu. Halo favor hare regulamentu konaba total licenca.')
						else:
							messages.error(request, f'Loron nebe hili barak liu. Halo favor hare regulamentu konaba total licenca.')
					else:
						messages.error(request,'Data hahu laiha periode ida ne nia laran. Halo Favor Kontakto Rekurso Humano hodi Kria Uluk Periode Licensa Foun!')
			else: form = LeaveForm()
		else:
			messages.error(request, 'Periode ba Lisensa seidauk determina')
			return redirect('leave-hr-leave-record', c_emp.hashed)
		context = {
			'group': group, 'c_emp': c_emp, 'empdiv': empdiv, 'form': form, 'page': 'apply',
			'title': 'Formulario Aplika Licensa', 'legend': 'Formulario Aplika Licensa'
		}
		return render(request, 'leave/form.html', context)
	else:
		return redirect('leave-hr-leave-record', c_emp.hashed)


@login_required
def hrLeaveSendProcess(request, hashid, leave):
	c_emp = get_object_or_404(Employee, hashed=hashid)
	group = c_emp.employeeuser.user.groups.all()[0].name
	objects = get_object_or_404(Leave, hashed=leave)
	empdiv = CurEmpDivision.objects.get(employee=c_emp)
	if group == "staff" or group == "hr":
		check_ekipa = EmpPosition.objects.filter(Q(department__isnull=False)&Q(department=empdiv.department), is_active=True, is_manager=True).exists()
		check_div = EmpPosition.objects.filter(Q(unit__isnull=False)&Q(unit=empdiv.unit), is_active=True, is_manager=True).exists()
		if check_ekipa and check_div or check_ekipa:
			objects.is_send =  True
			objects.is_lock =  True
			objects.is_finish =  False
			objects.save()
			messages.success(request, f'Manda ona.')
			return redirect('leave-hr-leave-record', c_emp.hashed)
		elif check_div:
			objects.is_send_to_div =  True
			objects.is_lock =  True
			objects.is_finish =  False
			objects.save()
			messages.success(request, f'Manda ona.')
			return redirect('leave-hr-leave-record', c_emp.hashed)
		else:
			messages.error(request, "Funsionario Refere Laiha Chefe Ekipa no Chefe Divizaun, Halo favor kontakto Rekurso Humano")
			return redirect('leave-hr-leave-record', c_emp.hashed)


@login_required
@allowed_users(allowed_roles=['admin','hr'])
def hrLeaveUpdateRecord(request):
	employee = Employee.objects.filter(status_id=1)
	for emp in employee:
		contract = Contract.objects.filter(employee=emp, is_active=True).exists()
		if contract:
			leave_type = None
			if emp.sex == 'Male':
				leave_type = LeaveType.objects.exclude(pk=4)
			else:
				leave_type = LeaveType.objects.all()
			leave_period = LeavePeriod.objects.filter(is_active=True, employee=emp).last()
			check_last_period = LeavePeriod.objects.filter(pk__lt=leave_period.pk, employee=emp).last()
			last_count_period = LeaveCount.objects.filter(employee=emp, period=check_last_period, leave_type_id=1).last()
			min_month = f'{leave_period.start_year.year}-{leave_period.start_month.code}'
			max_month = f'{leave_period.end_year.year}-{leave_period.end_month.code}'
			months = pd.period_range(min_month, max_month, freq='M')
			check_leave_count = LeaveCount.objects.filter(period=leave_period, employee=emp).exists()
			if check_leave_count == False:
				i = 0
				while i < len(months):
					leave_check = LeaveCount.objects.filter(employee=emp, period=leave_period).last()
					m = months[i]
					get_m = get_object_or_404(Mnth, code=m.month)
					get_y = get_object_or_404(Yr, year=m.year)
					if leave_check:
						for lt1 in leave_type:
							if lt1.pk == 1:
								lastleaveal = LeaveCount.objects.filter(employee=emp, period=leave_period, leave_type_id=1).last()
								earn_per_month = round(float(lt1.total/12),2)
								date_period_string = f"{m.year}-{m.month}-{leave_period.start_date.day}"
								date_period = dt.strptime(date_period_string,"%Y-%m-%d").date()
								lcnew = LeaveCount.objects.create(
									period = leave_period,
									employee=emp,
									leave_type=lt1,
									month = get_m,
									year = get_y,
									update_date = date_period,
									leave_earn = earn_per_month
								)
								lcnew.balance = round(float(lastleaveal.balance) + float(lcnew.leave_earn) - float(lcnew.taken),2)
								lcnew.total_earn = round(float(lastleaveal.prov_total_earn) + float(lcnew.leave_earn),1)
								lcnew.total_balance = round(float(lcnew.total_earn) - float(lcnew.total_taken),1)
								lcnew.balance_carry = lcnew.total_balance
								lcnew.total_balance_leave = float(lt1.total) - float(lcnew.total_balance)
								lcnew.prov_total_earn = float(lastleaveal.prov_total_earn) + float(earn_per_month)
								lcnew.balance_month = round(float(lastleaveal.balance) - float(lcnew.taken),2)
								lcnew.save()
					else:
						for lt in leave_type:
							if lt.pk == 1:
								# work: Check Balansu iha Periode Kotuk
								date_period_string = f"{m.year}-{m.month}-{leave_period.start_date.day}"
								date_period = dt.strptime(date_period_string,"%Y-%m-%d").date()
								if check_last_period:
									earn_per_month = round(float(lt.total/12),2)

									lc = LeaveCount.objects.create(
										period = leave_period,
										employee=emp,
										leave_type=lt,
										month = get_m,
										year = get_y,
										update_date=date_period,
										leave_earn = earn_per_month
									)
									lc.balance = round(float(lc.leave_earn) - float(lc.taken),2)
									lc.total_earn = round(float(lc.leave_earn) + float(last_count_period.total_balance),2)
									lc.total_balance = round(float(lc.total_earn) - float(lc.total_taken),2)
									lc.balance_carry = lc.total_balance
									lc.total_balance_leave = float(lt.total) - float(lc.total_balance)
									lc.prov_total_earn = lc.total_earn
									lc.balance_month = round(float(lc.balance) - float(lc.taken),2)
									lc.save()
								else:
									earn_per_month = round(float(lt.total/12),2)
									# balance_per_month = round(float(earn_per_month) - float(taken),2)

									lc = LeaveCount.objects.create(
										period = leave_period,
										employee=emp,
										leave_type=lt,
										month = get_m,
										year = get_y,
										update_date=date_period,
										leave_earn = earn_per_month
									)
									lc.balance = round(float(lc.leave_earn) - float(lc.taken),2)
									lc.total_earn = round(float(lc.leave_earn),2)
									lc.total_balance = round(float(lc.balance),2)
									lc.balance_carry = lc.total_balance
									lc.total_balance_leave = float(lt.total) - float(lc.total_balance)
									lc.prov_total_earn = lc.leave_earn
									lc.balance_month = round(float(lc.balance) - float(lc.taken),2)
									lc.save()
							else:
								lcall = LeaveCount.objects.create(
									period = leave_period,
									employee=emp,
									leave_type=lt,
									year = get_y,
								)
								lcall.total_balance = float(lt.total)
								lcall.save()
					i+=1
	messages.success(request, f'Susesu Update Record')
	return redirect('leave-hr-app-raw-list')
	

def update_leave_count(alldata, month):

	for obj in alldata:
		print(obj)



# work: Work Here
@login_required
@allowed_users(allowed_roles=['admin','hr'])
def hrLeavePeriodAdd(request):
	group = request.user.groups.all()[0].name
	employee = Employee.objects.filter(status_id=1)
	
	if request.method == 'POST':
		form = HRPeriodForm(request.POST)
		if form.is_valid():
			instance = form.save(commit=False)
			start_year = form.cleaned_data.get('start_year')
			end_year = form.cleaned_data.get('end_year')
			if start_year.year == end_year.year:
				messages.error(request, "Tinan Hahu ho Tinan Remata labele hanesan tinan remata !")
			elif start_year.year >= end_year.year:
				messages.error(request, "Tinan Hahu Labele boot liu Tinan Remata")
			else:
				for emp in employee:
					lp = LeavePeriod.objects.filter(start_year=start_year, end_year=end_year, employee=emp).exists()
					if  lp == False:
						contract = Contract.objects.filter(employee=emp, is_active=True).last()
						if contract:
							allleavep = LeavePeriod.objects.filter(employee=emp)
							for obj in allleavep:
								obj.is_active = False
								obj.save()
							# instance.is_active = True
							# instance.employee = emp
							date_string = f'{start_year}-{contract.start_date.month}-{contract.start_date.day}'
							start_period = dt.strptime(date_string, "%Y-%m-%d").date()
							next_year = pd.date_range(start=start_period, end=start_period+pd.DateOffset(years=1), freq='M')
							end_period = next_year.max()
							LeavePeriod.objects.create(
								employee=emp, \
								start_month = get_object_or_404(Mnth, code=start_period.month),\
								end_month = get_object_or_404(Mnth, code=end_period.month), \
								start_year = get_object_or_404(Yr, year=start_period.year), \
								end_year = get_object_or_404(Yr, year=end_period.year), \
								start_date = start_period,\
								end_date = end_period,\
								is_active = True
							)
					else:
						messages.error(request, f'Funsionario {emp} Seidauk iha Kontrato')
				messages.success(request, 'Susesu Aumenta Periode Licensa')
				return redirect('leave-hr-app-raw-list')
	else: form = HRPeriodForm()
	context = {
		'group': group,  'form':form, 'page':'period',
		'title': 'Set Period Leave', 'legend': 'Set Period Leave'
	}
	return render(request, 'leave/form2.html', context)

@login_required
@allowed_users(allowed_roles=['admin','hr'])
def hrLeaveEmpPeriodAdd(request, hashid):
	group = request.user.groups.all()[0].name
	employee = get_object_or_404(Employee, hashed=hashid)
	contract = Contract.objects.filter(employee=employee, is_active=True).last()
	period = LeavePeriod.objects.filter(employee=employee)
	if request.method == 'POST':
		form = HRPeriodForm(request.POST)
		if form.is_valid():
			instance = form.save(commit=False)
			start_year = form.cleaned_data.get('start_year')
			end_year = form.cleaned_data.get('end_year')
			if start_year.year == end_year.year:
				messages.error(request, "Tinan Hahu ho Tinan Remata labele hanesan")
			elif start_year.year >= end_year.year:
				messages.error(request, "Tinan Hahu labele boot liu Tinan Remata")
			else:
				allleavep = LeavePeriod.objects.filter(employee=employee)
				for obj in allleavep:
					obj.is_active = False
					obj.save()
				instance.is_active = True
				instance.employee = employee
				date_string = f'{start_year}-{contract.start_date.month}-{contract.start_date.day}'
				start_period = dt.strptime(date_string, "%Y-%m-%d").date()
				next_year = pd.date_range(start=start_period, end=start_period+pd.DateOffset(years=1), freq='M')
				end_period = next_year.max()
				instance.start_month = get_object_or_404(Mnth, code=start_period.month)
				instance.end_month = get_object_or_404(Mnth, code=end_period.month)
				instance.start_year = get_object_or_404(Yr, year=start_period.year)
				instance.end_year = get_object_or_404(Yr, year=end_period.year)
				instance.start_date = start_period
				instance.end_date = end_period
				instance.save()
				messages.success(request, 'Susesu Aumenta Periode Licensa')
				return redirect('leave-hr-leave-record', hashid)
	else: form = HRPeriodForm()
	context = {
		'group': group, 'form':form, 'page': 'add-emp-record', 'hashid': hashid,
		'title': 'Kria Periode Licensa', 'legend': 'Kria Periode Licensa', 'period': period
	}
	return render(request, 'leave/form2.html', context)

@login_required
@allowed_users(allowed_roles=['admin','hr'])
def hrLeaveEmpUpdatePeriodMonth(request, hashid):
	today = dt.today().date()
	group = request.user.groups.all()[0].name
	employee = get_object_or_404(Employee, hashed=hashid)
	lt = get_object_or_404(LeaveType, pk=1)
	leave_period = LeavePeriod.objects.filter(is_active=True, employee=employee).last()
	leave_count_last = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type_id=1).last()
	get_m = get_object_or_404(Mnth, code=today.month)
	get_y = get_object_or_404(Yr, year=today.year)
	if leave_count_last:
		earn_per_month = round(float(lt.total/12),2)

		lcnew = LeaveCount.objects.create(
			period = leave_period,
			employee=employee,
			leave_type=lt,
			month = get_m,
			year = get_y,
			leave_earn = earn_per_month
		)
		lcnew.balance = round(float(leave_count_last.balance) + float(lcnew.leave_earn) - float(lcnew.taken),2)
		lcnew.total_earn = round(float(leave_count_last.prov_total_earn) + float(lcnew.leave_earn),1)
		lcnew.total_balance = round(float(lcnew.total_earn) - float(lcnew.total_taken),1)
		lcnew.balance_carry = lcnew.total_balance
		lcnew.total_balance_leave = float(lt.total) - float(lcnew.total_balance)
		lcnew.prov_total_earn = float(leave_count_last.prov_total_earn) + float(earn_per_month)
		lcnew.save()
		messages.success(request, 'Susesu kria Record iha fulan ida ne')
		return redirect('leave-hr-leave-record', hashid)
	else:
		messages.error(request, 'Deskulpa Laiha Licensa iha Fulan kotuk')
		return redirect('leave-hr-leave-record', hashid)



@login_required
@allowed_users(allowed_roles=['hr','hr_s'])
def hrLeaveUpdateAttBack(request, hashid):
	group = request.user.groups.all()[0].name
	leave = get_object_or_404(Leave, hashed=hashid)
	emp = leave.employee
	per = pd.date_range(start=leave.start_date, end=leave.end_date, freq='B')
	if leave.leave_type.pk == 1:
		attstatus = get_object_or_404(AttendanceStatus, pk=6)
	elif leave.leave_type.pk == 2:
		attstatus = get_object_or_404(AttendanceStatus, pk=3)
	elif leave.leave_type.pk == 3:
		attstatus = get_object_or_404(AttendanceStatus, pk=12)
	elif leave.leave_type.pk == 4:
		attstatus = get_object_or_404(AttendanceStatus, pk=7)
	elif leave.leave_type.pk == 5:
		attstatus = get_object_or_404(AttendanceStatus, pk=8)
	for i in per:
		newid, hashedid = getnewid(Attendance)
		y = leave.start_date.year
		m = leave.start_date.month
		year = get_object_or_404(Year, year=y)
		month = get_object_or_404(Month, pk=m)
		created = Attendance.objects.create(
		id = newid,
		unit = emp.curempdivision.unit,
		employee = emp,
		year = year,
		month = month,
		date = i,
		status_am = attstatus,
		status_pm = attstatus,
		datetime=timezone.now(),
		user=request.user,
		hashed = hashedid)
		leave.is_update = True
		leave.save()


	messages.success(request, 'Susesu Altera')
	return redirect('leave-hr-app-detail', hashid)
@login_required
@allowed_users(allowed_roles=['hr','hr_s'])
def hrLeaveUpdateAtt(request, hashid, hashid2):
	group = request.user.groups.all()[0].name
	employee = get_object_or_404(Employee, hashed=hashid)
	leave_period = LeavePeriod.objects.filter(employee=employee,is_active=True).last()
	check_last_period = LeavePeriod.objects.filter(employee=employee,  pk__lt=leave_period.pk).last()
	last_count_period = LeaveCount.objects.filter(employee=employee, period=check_last_period, leave_type_id=1).last()
	min_month = f'{leave_period.start_year.year}-{leave_period.start_month.code}'
	max_month = f'{leave_period.end_year.year}-{leave_period.end_month.code}'
	months = pd.period_range(min_month, max_month, freq='M')
	leave = get_object_or_404(Leave, hashed=hashid2)


	leave_type = leave.leave_type
	days = leave.days
	days = round(float(days),2)

	if leave_type.pk == 1: earn_per_month = round(float(leave_type.total/12),2)
	else: earn_per_month = 0.00

	start_date = leave.start_date
	end_date = leave.end_date
	last_two_digits = str(days).split('.')[1][:2]
	last_two_digits = int(last_two_digits)

	min_month = f'{leave_period.start_year.year}-{leave_period.start_month.code}'
	max_month = f'{leave_period.end_year.year}-{leave_period.end_month.code}'
	date1 = pd.to_datetime(start_date, format="%Y %m").date()

	start_date_period = pd.to_datetime(min_month, format="%Y %m").date()
	end_date_period = pd.to_datetime(max_month, format="%Y %m").date()
	start_month = date1


	if days <= leave_type.total:
		# done: CHECK ANGKA LORON 0.5/1.0/1.5
		if last_two_digits == int(5) or last_two_digits == int(0):
			# done: CHECK DATA LEAVE IHA PERIODO NIA LARAN
			if start_date_period <= start_month <= end_date_period + datetime.timedelta(days=29):
				# done: CHECK LEAVE TYPE BA ANNUAL
				if leave_type.pk == 1:
					
					# done: CHECK RECORD IHA FULAN IDA NE'E
					
					current_leave_count = LeaveCount.objects.filter(period=leave_period, employee=employee,leave_type= leave_type, month__code=start_date.month, year__year=start_date.year).last()
					if current_leave_count:
						previous_leave_count = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type= leave_type, month__code__lt=start_date.month, year__year=current_leave_count.year.year).last()
					# done: CHECK KARIK LEAVE IHA FULAN KLARAN
					if previous_leave_count:
						current_leave_count.taken = round(float(current_leave_count.taken) + days,2)
						current_leave_count.total_taken = round(float(previous_leave_count.taken) + days)
						current_leave_count.balance = round(float(previous_leave_count.balance) + float(current_leave_count.leave_earn) - float(days),2 )
						current_leave_count.total_balance = round(float(current_leave_count.total_earn) - float(current_leave_count.total_taken),1)
						current_leave_count.balance_month = round(float(previous_leave_count.balance) - float(current_leave_count.taken),2)
						current_leave_count.save()



						#done: KRIA ATTENDANCE
						createAttendanceEmp(request, leave_type, Attendance, AttendanceStatus, employee, start_date, end_date, YAten, Maten, leave)							
					# done: CHECK KARIK LEAVE IHA FULAN PRIMEIRU
					else:
						current_leave_count.taken = round(float(current_leave_count.taken) + days,2)
						if check_last_period:
							current_leave_count.total_taken = current_leave_count.taken
							current_leave_count.total_earn = round(float(last_count_period.total_balance) + float(earn_per_month),1)
							current_leave_count.balance = round(float(last_count_period.total_balance) + float(earn_per_month) - float(current_leave_count.taken),2 )
							current_leave_count.total_balance = round(float(current_leave_count.total_earn) - float(current_leave_count.total_taken),1)
							current_leave_count.balance_month = round(float(current_leave_count.total_balance) - float(current_leave_count.taken),2)
							current_leave_count.save()
						else:
							current_leave_count.total_taken = current_leave_count.taken
							current_leave_count.balance = round(float(current_leave_count.leave_earn) - float(current_leave_count.taken),2 )
							current_leave_count.total_balance = round(float(current_leave_count.total_earn) - float(current_leave_count.total_taken),1)
							current_leave_count.balance_month = round(float(current_leave_count.balance) - float(current_leave_count.taken),2)
							current_leave_count.save()

						# done: KRIA ATTENDANCE
						createAttendanceEmp(request, leave_type, Attendance, AttendanceStatus, employee, start_date, end_date, YAten, Maten, leave)


					i = 0
					# done: UPDATE LEAVE RECORD
					while i < len(months):
						m = months[i]
						if i == int(0):
							# done: FULAN PRIMEIRO
							if check_last_period:
								first_leave_count = LeaveCount.objects.filter(employee=employee,month__code=m.month, year__year=m.year, period=leave_period,leave_type=leave_type).first()
								first_leave_count.balance = round(float(last_count_period.total_balance) + float(earn_per_month) -  float(first_leave_count.taken),2)
								first_leave_count.total_earn = round(float(first_leave_count.leave_earn) + float(last_count_period.total_balance),2)
								first_leave_count.total_balance = round(float(first_leave_count.total_earn) - float(first_leave_count.total_taken),2)
								first_leave_count.balance_carry = first_leave_count.total_balance
								first_leave_count.total_balance_leave = float(leave_type.total) - float(first_leave_count.total_balance)
								first_leave_count.prov_total_earn = first_leave_count.total_earn
								first_leave_count.balance_month = round(float(first_leave_count.balance) - float(first_leave_count.taken),2)
								first_leave_count.save()

							else:
								if leave_period.balance_carry > 0.00:
									first_leave_count = LeaveCount.objects.filter(employee=employee,month__code=m.month, year__year=m.year, period=leave_period,leave_type=leave_type).first()
									first_leave_count.balance = round(float(leave_period.balance_carry) + float(first_leave_count.leave_earn) - float(first_leave_count.taken),2)
									first_leave_count.total_earn = round(float(leave_period.balance_carry) + float(first_leave_count.leave_earn),2)
									first_leave_count.total_balance = round( float(first_leave_count.balance),2)
									first_leave_count.balance_carry = first_leave_count.total_balance
									first_leave_count.total_balance_leave = float(leave_period.balance_carry) + float(leave_type.total) - float(first_leave_count.total_balance)
									first_leave_count.prov_total_earn =  float(leave_period.balance_carry) + float(first_leave_count.leave_earn)
									first_leave_count.balance_month = round(float(first_leave_count.balance) - float(first_leave_count.taken),2)
									first_leave_count.save()
								else:
									first_leave_count = LeaveCount.objects.filter(employee=employee,month__code=m.month, year__year=m.year, period=leave_period,leave_type=leave_type).first()
									first_leave_count.balance = round(float(first_leave_count.leave_earn) - float(first_leave_count.taken),2)
									first_leave_count.total_earn = round(float(first_leave_count.leave_earn),2)
									first_leave_count.total_balance = round(float(first_leave_count.balance),2)
									first_leave_count.balance_carry = first_leave_count.total_balance
									first_leave_count.total_balance_leave = float(leave_type.total) - float(first_leave_count.total_balance)
									first_leave_count.prov_total_earn = first_leave_count.leave_earn
									first_leave_count.balance_month = round(float(first_leave_count.balance) - float(first_leave_count.taken),2)
									first_leave_count.save()

						else:
							# done: FULAN TUIR MAI
							prev_mont = m.asfreq("M", "S") - 1
							lccurrent = get_object_or_404(LeaveCount, employee=employee, period=leave_period, leave_type=leave_type,  year__year=m.year, month__code=m.month)
							lclast = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type=leave_type,  year__year=prev_mont.year, month__code=prev_mont.month).first()

							if leave_period.balance_carry > 0.00:
								lccurrent.balance = round(float(lclast.balance) + float(lccurrent.leave_earn) - float(lccurrent.taken),2)
								lccurrent.total_earn = round(float(lclast.prov_total_earn) + float(lccurrent.leave_earn),1)
								lccurrent.total_taken = round(float(lclast.total_taken) + float(lccurrent.taken),2)
								lccurrent.total_balance = round(float(lccurrent.total_earn) - float(lccurrent.total_taken),1)
								lccurrent.balance_carry = lccurrent.total_balance
								lccurrent.total_balance_leave = float(leave_type.total) - float(lccurrent.total_balance)
								lccurrent.prov_total_earn = float(lclast.prov_total_earn) + float(earn_per_month)
								lccurrent.balance_month = round(float(lclast.balance) - float(lccurrent.taken),2)
								lccurrent.save()
							else:
								lccurrent.balance = round(float(lclast.balance) + float(lccurrent.leave_earn) - float(lccurrent.taken),2)
								lccurrent.total_earn = round(float(lclast.prov_total_earn) + float(lccurrent.leave_earn),1)
								lccurrent.total_taken = round(float(lclast.total_taken) + float(lccurrent.taken),2)
								lccurrent.total_balance = round(float(lccurrent.total_earn) - float(lccurrent.total_taken),1)
								lccurrent.balance_carry = lccurrent.total_balance
								lccurrent.total_balance_leave = float(leave_type.total) - float(lccurrent.total_balance)
								lccurrent.prov_total_earn = float(lclast.prov_total_earn) + float(earn_per_month)
								lccurrent.balance_month = round(float(lclast.balance) - float(lccurrent.taken),2)
								lccurrent.save()

						i+=1

					# done: KRIA LEAVE

				# done: CHECK KARIK LEAVE LAOS ANNUAL LEAVE
				else:

					lcount = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type=leave_type, month__isnull=False)
					getmonth = get_object_or_404(Mnth, code=start_date.month)
					getyear = get_object_or_404(Yr, year=start_date.year)

					# done: CHECK LEAVE COUNT NEBE IHA ONA MAIBE FULAN LAIHA
					if lcount:
						previous_leave_count = LeaveCount.objects.filter(period=leave_period, employee=employee,leave_type=leave_type).last()
						month_leave_count = LeaveCount.objects.filter(period=leave_period, employee=employee,leave_type= leave_type, month__code=start_date.month, year__year=start_date.year).last()
						# work: Leave Nebe Iha ona
						if days <= previous_leave_count.total_balance:
							if month_leave_count:
								month_leave_count.taken = round(float(month_leave_count.taken) + days, 2)
								month_leave_count.total_taken = round(float(previous_leave_count.total_taken) +  days,2)
								month_leave_count.total_balance = round(float(leave_type.total) - float(month_leave_count.total_taken),2)
								month_leave_count.save()
								createAttendanceEmp(request, leave_type, Attendance, AttendanceStatus, employee, start_date, end_date, YAten, Maten, leave)
								messages.success(request, 'Susesu Altera')
								return redirect('leave-hr-app-detail', hashid2)
							else:
								lc = LeaveCount.objects.create(
									period = leave_period,
									employee = employee,
									leave_type = leave_type,
									year = getyear,
									month = getmonth

								)
								lc.taken = days
								lc.total_taken = round(float(previous_leave_count.total_taken) + float(lc.taken),2)
								lc.total_balance = round(float(leave_type.total) - float(lc.total_taken),2)
								lc.save()

								createAttendanceEmp(request, leave_type, Attendance, AttendanceStatus, employee, start_date, end_date, YAten, Maten, leave)

								messages.success(request, 'Susesu Altera')
								return redirect('leave-hr-app-detail', hashid2)
						else:
							messages.error(request, 'Loron nebe hili barak liu. Halo favor altera loron nebe ita prienche')
					# done: CHECK LEAVE COUNT NEBE IHA NO FULAN ONA IHA
					else:
						lcount2 = LeaveCount.objects.filter(employee=employee, period=leave_period, leave_type=leave_type)
						lcount2.update(
							month=getmonth,
							year=getyear,
							taken = days,
							total_taken = days,
							total_balance = round(float(leave_type.total) - float(days),2)
						)

						# KRIA ATTENDANCE
						createAttendanceEmp(request, leave_type, Attendance, AttendanceStatus, employee, start_date, end_date, YAten, Maten, leave)

						messages.success(request, 'Susesu Altera')
						return redirect('leave-hr-app-detail', hashid2)
				
			# done: ERROR KARIK DATA HAHU LAIHA PERIODE NIA LARAN
			else:
				messages.error(request,'Data hahu laiha periode ida ne nia laran!!')
		else:
			messages.error(request,'Total loron nebe ita prienche latuir formato!!')
	else:
		messages.error(request, f'Loron nebe hili barak liu. Halo favor hare regulamentu konaba total licenca.')
	
	leave.hr_confirm = True
	leave.save()


	messages.success(request, 'Susesu Altera')
	return redirect('leave-hr-app-detail', hashid2)



# Generated by Django 4.0 on 2022-09-21 12:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='taskdelegate',
            name='date',
        ),
    ]



# Generated by Django 3.2 on 2022-12-26 14:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0054_auto_20221225_2232'),
    ]

    operations = [
        migrations.AddField(
            model_name='leave',
            name='description',
            field=models.CharField(blank=True, max_length=250, null=True, verbose_name='Deskrisaun Licensa'),
        ),
    ]



# Generated by Django 4.0 on 2022-09-25 02:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0017_remove_leavecount_datetime_remove_leavecount_hashed_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='leavecount',
            name='balance',
            field=models.IntegerField(blank=True, null=True, verbose_name='Balansu'),
        ),
    ]



from django.urls import path
from . import views

urlpatterns = [
	path('emp/leave/', views.APIEmpLeave.as_view()),
	path('emp/leave/type/', views.APIEmpLeaveType.as_view()),
	path('emp/leave/type/2/<int:year>/', views.APIEmpLeaveType2.as_view()),
	path('emp/leave/type/2/<int:year>/<int:month>/', views.APIEmpLeaveType3.as_view()),
	path('emp/leave/type/unit/<int:unit>/', views.APIEmpLeaveUnit.as_view()),
	path('emp/leave/type/month/<int:month>/', views.APIEmpLeaveMonth.as_view()),
	path('emp/leave/type/tri/<int:tri>/', views.APIEmpLeaveTri.as_view()),
	path('emp/leave/type/year-unit/<int:year>/<int:unit>/', views.APIEmpLeaveYearUnit.as_view()),
	path('emp/leave/type/year-month/<int:year>/<int:month>/', views.APIEmpLeaveYearMonth.as_view()),
	path('emp/leave/type/year-tri/<int:year>/<int:tri>/', views.APIEmpLeaveYearTri.as_view()),
	path('emp/leave/type/month-unit/<int:month>/<int:unit>/', views.APIEmpLeaveMonthUnit.as_view()),
	path('emp/leave/type/year-month-unit/<int:year>/<int:month>/<int:unit>/', views.APIEmpLeaveYearMonthUnit.as_view()),
	path('emp/leave/type/year-tri-unit/<int:year>/<int:tri>/<int:unit>/', views.APIEmpLeaveYearTriUnit.as_view()),
]


from django.shortcuts import render, get_object_or_404, redirect
from django.contrib.auth.decorators import login_required
from settings_app.decorators import allowed_users
from django.db.models import Sum, Count, Q
from leave.models import Leave, LeaveCount, LeaveDE, LeaveDelegate, LeaveHR, LeaveType, LeaveUnit, LeaveDep, LeavePeriod
from settings_app.user_utils import c_dep, c_staff, c_unit
from django.contrib import messages
from datetime import datetime
from settings_app.models import IPGInfo
import xhtml2pdf.pisa as pisa
from django.http import HttpResponse
from django.template.loader import get_template
from employee.models import EmpSignature

@login_required
@allowed_users(allowed_roles=['staff','dep', 'hr'])
def sLeaveList(request):
	group = request.user.groups.all()[0].name
	c_emp = c_staff(request.user)
	objects = Leave.objects.filter(employee=c_emp).all().order_by('-start_date')
	context = {
		'group': group, 'objects': objects,
		'title': 'Lista Licensa', 'legend': 'Lista Licensa'
	}
	return render(request, 'leave/s_apply_list.html', context)

@login_required
@allowed_users(allowed_roles=['staff','dep', 'hr'])
def sLeaveDetail(request, hashid):
	group = request.user.groups.all()[0].name
	leave = get_object_or_404(Leave, hashed=hashid)
	leavedel = LeaveDelegate.objects.filter(leave=leave).first()
	leavedep = LeaveDep.objects.filter(leave=leave).first()
	leaveunit = LeaveUnit.objects.filter(leave=leave).first()
	leavehr = LeaveHR.objects.filter(leave=leave).first()
	leavede = LeaveDE.objects.filter(leave=leave).first()
	context = {
		'group': group, 'leave': leave, 'leavedel': leavedel, 'leaveunit': leaveunit,
		'leavehr': leavehr, 'leavede': leavede,'leavedep':leavedep,
		'title': 'Detalha Licensa', 'legend': 'Detalha Licensa'
	}
	return render(request, 'leave/s_apply_detail.html', context)

@login_required
@allowed_users(allowed_roles=['staff','dep'])
def LeaveTerminate(request, hashid):
	group = request.user.groups.all()[0].name
	leave = get_object_or_404(Leave, hashed=hashid)
	leave.is_done = True
	leave.save()
	messages.success(request, 'Susesu Termina Licensa')
	if group == 'staff':
		return redirect('leave-s-list')
	if group == 'dep':
		return redirect('leave-dep-list')
###
@login_required
def sLeaveBalCheck(request):
	group = request.user.groups.all()[0].name
	today = datetime.today().date()
	page = ''
	c_emp = []
	if group == "staff":
		c_emp = c_staff(request.user)
		page = 'cstaff'
	elif group == "dep":
		c_emp, _ = c_dep(request.user)
		page = 'cdep'
	elif group == "unit":
		c_emp, _ = c_unit(request.user)
		page = 'cunit'
	elif group == "de":
		c_emp, _ = c_unit(request.user)
		page = 'cde'
	elif group == "deputy":
		c_emp, _ = c_unit(request.user)
		page = 'cdeputy'
	else:
		c_emp, _ = c_unit(request.user)
	period = LeavePeriod.objects.filter(employee=c_emp, is_active=True).last()
	if period:
		if c_emp.sex == 'Female':
			leavetypes = LeaveType.objects.all()
			objects = []
		else:
			leavetypes = LeaveType.objects.exclude(pk=4)
			objects = []

		for i in leavetypes:
			a = LeaveCount.objects.filter(employee=c_emp, period__is_active=True, update_date__lte=today, period__employee=c_emp,  leave_type=i).last()
			objects.append([i,a])
		context = {
			'group': group, 'c_emp': c_emp, 'objects': objects, 'page': page,
			'title': 'Lista Balansu Licensa', 'legend': 'Lista Balansu Licensa'
		}
		return render(request, 'leave/bal_s_list.html', context)
	else:
		messages.error(request, 'Periode Licensa seidauk kria. Halo Favor Kontakto Rekurso Humano')
		if group == "staff": return redirect('leave-s-list')
		if group == "dep": return redirect('leave-dep-list')
		elif group == "unit": return redirect('leave-c-list')
		elif group == "de": return redirect('leave-de-list')
		elif group == "deputy": return redirect('leave-de-list')
		elif group == "hr": return redirect('leave-hr-list')


@login_required
@allowed_users(allowed_roles=['staff','unit', 'dep', 'deputy', 'de'])
def sLeavePDF(request, hashid):
	group = request.user.groups.all()[0].name
	leave = get_object_or_404(Leave, hashed=hashid)
	cgroup = leave.user.groups.all()[0].name
	leavedep = LeaveDep.objects.filter(leave=leave).first()
	leaveunit = LeaveUnit.objects.filter(leave=leave).first()
	leavehr = LeaveHR.objects.filter(leave=leave).first()
	leavede = LeaveDE.objects.filter(leave=leave).first()
	ipginfo = IPGInfo.objects.filter(is_active=True).last()
	response = HttpResponse(content_type='application/pdf')
	if leavede.user:
		signature = EmpSignature.objects.filter(employee=leavede.user.employeeuser.employee).last()
	if leaveunit.user:
		signature = EmpSignature.objects.filter(employee=leaveunit.user.employeeuser.employee).last()

	response['Content-Disposition'] = f'inline; filename="Leave_ON_{leave.start_date.day}_{leave.start_date.month}_{leave.start_date.year}.pdf"'
	template = get_template('pdf/leave.html')
	host_url = request.get_host()
	path = f'{host_url}/media/{ipginfo.cop}'
	signature_path = ''
	if signature:
		signature_path = f'{host_url}/media/{signature.image}'
	context = {
		'mpm_path': 'main/static/main/images/mpm.png',
		'ipg_path': 'https://'+path,
		'signature_path': 'https://'+signature_path,
		'title': 'PDF Licensa', 'cgroup':cgroup, 
		'leavedep':leavedep, 'leaveunit':leaveunit, 'leavehr':leavehr,
		'leavede':leavede, 'leave':leave, 'ipginfo':ipginfo
	}
	html = template.render(context)

	pisaStatus = pisa.CreatePDF(html, dest=response)

	return response


from django.shortcuts import render, get_object_or_404
from django.contrib.auth.decorators import login_required
from settings_app.decorators import allowed_users
from django.db.models import Sum, Count, Q
from leave.models import Leave, LeaveDE, LeaveDelegate, LeaveHR, LeaveUnit
from settings_app.user_utils import c_unit

@login_required
@allowed_users(allowed_roles=['unit'])
def cLeaveVerList(request):
	group = request.user.groups.all()[0].name
	emp_unit, unit = c_unit(request.user)
	objects = Leave.objects.filter(
		Q((Q(employee__curempdivision__department__unit=unit)|\
        Q(employee__curempdivision__unit=unit)),dep_send=True, is_finish=False)|
		Q((Q(employee__curempdivision__department__unit=unit)|\
        Q(employee__curempdivision__unit=unit)),is_send_to_div=True, is_finish=False)

	).all().order_by('-start_date')
	context = {
		'group': group, 'objects': objects,
		'title': 'Lista Licensa hodi Aprova', 'legend': 'Lista Licensa hodi Aprova'
	}
	return render(request, 'leave/c_ver_list.html', context)

@login_required
@allowed_users(allowed_roles=['unit'])
def cLeaveVerDetail(request, hashid):
	group = request.user.groups.all()[0].name
	leave = get_object_or_404(Leave, hashed=hashid)
	leavedel = LeaveDelegate.objects.filter(leave=leave).first()
	leaveunit = LeaveUnit.objects.filter(leave=leave).first()
	context = {
		'group': group, 'leave': leave, 'leavedel': leavedel, 'leaveunit': leaveunit,
		'title': 'Detalha Licensa', 'legend': 'Detalha Licensa'
	}
	return render(request, 'leave/c_ver_detail.html', context)
###
@login_required
@allowed_users(allowed_roles=['unit', 'deputy'])
def cLeaveList(request):
	group = request.user.groups.all()[0].name
	c_emp, unit = c_unit(request.user)
	objects = Leave.objects.filter(employee=c_emp).all().order_by('-start_date')
	context = {
		'group': group, 'objects': objects,
		'title': 'Lista Licensa', 'legend': 'Lista Licensa'
	}
	return render(request, 'leave/c_apply_list.html', context)

@login_required
@allowed_users(allowed_roles=['unit', 'deputy'])
def cLeaveDetail(request, hashid):
	group = request.user.groups.all()[0].name
	leave = get_object_or_404(Leave, hashed=hashid)
	leavedel = LeaveDelegate.objects.filter(leave=leave).first()
	leaveunit = LeaveUnit.objects.filter(leave=leave).first()
	leavehr = LeaveHR.objects.filter(leave=leave).first()
	cgroup = leave.employee.employeeuser.user.groups.all()[0].name
	leavede = LeaveDE.objects.filter(leave=leave).first()
	context = {
		'group': group, 'leave': leave, 'leavedel': leavedel, 'leaveunit': leaveunit,
		'leavehr': leavehr, 'leavede': leavede, 'cgroup':cgroup,
		'title': 'Detalha Licensa', 'legend': 'Detalha Licensa'
	}
	return render(request, 'leave/c_apply_detail.html', context)


# Generated by Django 3.2 on 2022-12-11 02:51

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('leave', '0028_auto_20221211_0202'),
    ]

    operations = [
        migrations.CreateModel(
            name='LeaveDep',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('obs', models.CharField(blank=True, choices=[('Sim', 'Sim'), ('Lae', 'Lae')], max_length=3, null=True, verbose_name='Aprova')),
                ('reason', models.TextField(blank=True, null=True, verbose_name='Rajaun')),
                ('datetime', models.DateTimeField(null=True)),
                ('hashed', models.CharField(max_length=32, null=True)),
                ('leave', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='leavedep', to='leave.leave')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# Generated by Django 4.0 on 2022-09-25 03:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0019_leave_s_is_delegate'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='leave',
            name='s_is_delegate',
        ),
    ]



# Generated by Django 4.0 on 2022-09-23 05:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0007_leavetype_total'),
    ]

    operations = [
        migrations.RenameField(
            model_name='leave',
            old_name='de_confirm',
            new_name='de_approve',
        ),
        migrations.RenameField(
            model_name='leave',
            old_name='dep_confirm',
            new_name='hr_confirm',
        ),
        migrations.RenameField(
            model_name='leave',
            old_name='dep_send',
            new_name='hr_send',
        ),
    ]



# Generated by Django 4.0 on 2022-09-25 00:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0014_leavehr_leavede'),
    ]

    operations = [
        migrations.AlterField(
            model_name='leavede',
            name='obs',
            field=models.CharField(blank=True, choices=[('Sim', 'Sim'), ('Lae', 'Lae')], max_length=3, null=True, verbose_name='Aprova'),
        ),
        migrations.AlterField(
            model_name='leavedelegate',
            name='obs',
            field=models.CharField(blank=True, choices=[('Sim', 'Sim'), ('Lae', 'Lae')], max_length=3, null=True, verbose_name='Simu Delegasaun'),
        ),
        migrations.AlterField(
            model_name='leavehr',
            name='obs',
            field=models.CharField(blank=True, choices=[('Sim', 'Sim'), ('Lae', 'Lae')], max_length=3, null=True, verbose_name='Certifica'),
        ),
        migrations.AlterField(
            model_name='leaveunit',
            name='obs',
            field=models.CharField(blank=True, choices=[('Sim', 'Sim'), ('Lae', 'Lae')], max_length=3, null=True, verbose_name='Aprova'),
        ),
    ]



from django.shortcuts import render, get_object_or_404
from django.contrib.auth.decorators import login_required
from settings_app.decorators import allowed_users
from django.db.models import Sum, Count, Q
from leave.models import Leave, LeaveDelegate, LeaveDep, LeaveHR, LeaveUnit
from settings_app.user_utils import c_staff, c_dep
from employee.models import CurEmpDivision

@login_required
@allowed_users(allowed_roles=['admin','hr'])
def LeaveList(request):
	group = request.user.groups.all()[0].name
	c_emp = c_staff(request.user)
	objects = Leave.objects.filter(employee=c_emp).all().order_by('-start_date')
	context = {
		'group': group, 'objects': objects,
		'title': 'Lista Licensa', 'legend': 'Lista Licensa'
	}
	return render(request, 'leave/list.html', context)

@login_required
@allowed_users(allowed_roles=['admin','hr'])
def LeaveDetail(request, hashid):
	group = request.user.groups.all()[0].name
	objects = get_object_or_404(Leave, hashed=hashid)
	context = {
		'group': group, 'objects': objects,
		'title': 'Detalha Licensa', 'legend': 'Detalha Licensa'
	}
	return render(request, 'leave/detail.html', context)
###
@login_required
# @allowed_users(allowed_roles=['staff'])
def LeaveDelegList(request):
	group = request.user.groups.all()[0].name
	c_emp = c_dep(request.user)
	objects = LeaveDelegate.objects.filter(employee=c_emp).all().order_by('-leave__start_date')
	context = {
		'group': group, 'objects': objects,
		'title': 'Lista Aplikasaun Licensa', 'legend': 'Lista Aplikasaun Licensa'
	}
	return render(request, 'leave/deleg_list.html', context)

# work: Leave DEP
@login_required
@allowed_users(allowed_roles=['dep'])
def LeaveDepList(request):
	group = request.user.groups.all()[0].name
	c_emp = c_staff(request.user)
	objects = []
	objects = Leave.objects.filter(employee=c_emp).all().order_by('-pk')
	context = {
		'group': group, 'objects': objects,
		'title': 'Lista Aplikasaun', 'legend': 'Lista Aplikasaun'
	}
	return render(request, 'leave/dep_list.html', context)

@login_required
@allowed_users(allowed_roles=['dep'])
def LeaveDepAprvList(request):
	group = request.user.groups.all()[0].name
	c_emp = c_staff(request.user)
	objects = []
	objects = LeaveDep.objects.filter(leave__employee__curempdivision__department=c_emp.curempdivision.department).all().order_by('-pk')
	context = {
		'group': group, 'objects': objects,
		'title': 'Lista Predido Aprovasaun Lisensa', 'legend': 'Lista Predido Aprovasaun Lisensa'
	}
	return render(request, 'leave/dep_req_list.html', context)

@login_required
@allowed_users(allowed_roles=['dep'])
def LeaveDepDetail(request, hashid):
	group = request.user.groups.all()[0].name
	c_emp = c_staff(request.user)
	empdiv = CurEmpDivision.objects.get(employee=c_emp)
	leavedep = get_object_or_404(LeaveDep, hashed=hashid)
	leave = leavedep.leave
	leavehr = get_object_or_404(LeaveHR, leave=leave)
	leaveunit = get_object_or_404(LeaveUnit, leave=leave)
	cgroup = leave.user.groups.all()[0].name
	context = {
		'group': group, 'leave': leave, 'leavedep': leavedep, 'empdiv':empdiv, 'leavehr':leavehr, 'leaveunit':leaveunit,
		'title': 'Detalha Pedido Aplikasaun Licensa', 'legend': 'Detalha Pedido Aplikasaun Licensa', 'cgroup':cgroup
	}
	return render(request, 'leave/dep_detail.html', context)



@login_required
# @allowed_users(allowed_roles=['staff'])
def LeaveDelegDetail(request, hashid):
	group = request.user.groups.all()[0].name
	leavedel = get_object_or_404(LeaveDelegate, hashed=hashid)
	leave = leavedel.leave
	context = {
		'group': group, 'leave': leave, 'leavedel': leavedel,
		'title': 'Delegasaun Servisu', 'legend': 'Delegasaun Servisu'
	}
	return render(request, 'leave/deleg_detail.html', context)


# Generated by Django 4.0 on 2022-09-23 05:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0005_rename_is_approve_leavedelegate_is_confirm'),
    ]

    operations = [
        migrations.AddField(
            model_name='leavedelegate',
            name='is_confirm2',
            field=models.BooleanField(default=False, null=True),
        ),
    ]



# Generated by Django 4.0 on 2022-09-23 06:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leave', '0008_rename_de_confirm_leave_de_approve_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='leave',
            old_name='is_lock',
            new_name='s_is_finish',
        ),
        migrations.RenameField(
            model_name='leave',
            old_name='is_send',
            new_name='s_is_lock',
        ),
        migrations.AddField(
            model_name='leave',
            name='s_is_send',
            field=models.BooleanField(default=False, null=True),
        ),
    ]
